#!/usr/bin/env python

import argparse
import pickle
import logging
import os
import re
import sys
import math

__author__ = 'sliu'

# pass 1, where contains stage 7 folders, 2,
# contain the stage 8 result, 4 contain genchallenge data final files

RMSD_PICKLE = 'RMSD.pickle'
FINAL_LOG = 'final.log'
CHALL_FINAL_LOG = FINAL_LOG
LMCSS = 'LMCSS'
SMCSS = 'SMCSS'
HI_TANIMOTO = 'hiTanimoto'
HI_RESAPO = 'hiResApo'
HI_RESHOLO = 'hiResHolo'
SUMMARY_TXT = 'summary.txt'
OVERALL_RMSD_PREFIX = 'Overall_RMSD_'
CSV_SUFFIX = '.csv'
NUMBER_BINS = 8
BINSIZE = 1

logger = logging.getLogger()


def check_case_number(log_file, phrase):
    """Counts number of occurrences of `phrase`
       in file path passed in.
    :param log_file: path to file to examine
    :param phrase: phrase to search for in each line of file
    :returns: count of occurrences as int upon success, -1 if
              there was an error or if `log_file` or 1phrase`
              is None
    """
    # TODO Need to find a better way then parsing final.log to get # candidates
    total_number = -1
    if log_file is None:
        logger.error('log file is None')
        return total_number

    if phrase is None:
        logger.error('phrase is None')
        return total_number

    log_f = None
    try:
        log_f = open(log_file, "r")
        total_number = 0
        for log_line in log_f:
            if phrase in log_line:
                total_number += 1
    except IOError:
        logger.exception('Caught exception attempting to load file: ' +
                         log_file)
    finally:
        if log_f is not None:
            log_f.close()
    return total_number


def get_dock_scores_as_list(pickle_file, ctype=LMCSS):
    """Get list of dock scores for candidate type `ctype`
       from pickle_file `pickle_file`
       The format of the pickle file should be
       { 'ligand': {'ctype': score}}
    :param pickle_file: pickle file to parse
    :param ctype: candidate type
    :returns: list of values, no checking on types could be strings
              ints, floats etc.
    """
    dock_scores = []
    if pickle_file is None:
        logger.error('Pickle file is None')
        return dock_scores

    if ctype is None:
        logger.error('Candidate Type is None')
        return dock_scores

    try:
        p_f = open(pickle_file, "rb")
        p_d = pickle.load(p_f)
        p_f.close()
        for ligand in p_d:
            try:
                value = p_d[ligand][ctype]
                dock_scores.append(value)
            except KeyError:
                logger.error('No ' + ctype +
                             ' for ' + ligand +
                             ' skipping...')
                continue
        return dock_scores
    except Exception:
        logger.exception('Caught Exception just returning empty list')
        return []


def get_list_of_stats(list_of_dock_scores):
    """Extract average for given type
       by getting scores for all ligands with scores
       of type `ctype` in `pickle_file`
       The format of the pickle file should be
       { 'ligand': {'ctype': score}}
    :param list_of_dock_scores: list of dock scores
    :returns: tuple (count, min, max, average, median)
    """
    if list_of_dock_scores is None or len(list_of_dock_scores) is 0:
        logger.error('No dock scores to generate stats from')
        return -1, -1, -1, -1, -1
    try:
        count = len(list_of_dock_scores)

        average = sum(list_of_dock_scores)/count
        if count == 1:
            median = list_of_dock_scores[0]
        else:

            sorted_list = sorted(list_of_dock_scores)
            if count % 2 == 0:
                median_index = count / 2
                median = sum(sorted_list[median_index-1:median_index+1])/2
            else:
                median = sorted_list[int(round(count/2))]
        return (count,
                min(list_of_dock_scores),
                max(list_of_dock_scores),
                average, median)
    except Exception:
        logger.exception('Caught Exception just returning -1')
        return -1, -1, -1, -1, -1


def get_histogram_of_dock_scores(list_of_dock_scores, binsize,
                                 number_bins):
    """creates histogram of dock scores
    :param list_of_dock_scores: list of dock scores
    :param binsize: size of each bin
    :param number_bins: number of bins. If set to say 4 then
                        bin 1 will be values between 0 and < binsize
                        bin 2 will be values between binsize & < binsize x 2
                        bin 3 will be values between binsize x 2 &
                                                     < binsize x 3
                        bin 4 will be values binsize x 3 and greater
    :returns list containing number of scores in each bin
    """
    if number_bins is None or number_bins <= 0:
        logger.error('number_bins is None or <= 0')
        return None
    if binsize <= 0:
        logger.error('binsize is <= 0')
        return None
    if list_of_dock_scores is None or len(list_of_dock_scores) is 0:
        logger.error('No dock scores to generate histogram from')
        return [0] * number_bins

    histo = [0] * number_bins
    for score in list_of_dock_scores:
        thebin = int(math.floor(score/binsize))
        if thebin < 0:
            thebin = 0

        if thebin >= number_bins:
            thebin = number_bins - 1
        histo[thebin] += 1
    return histo


def _get_pickle_paths(path_list):
    """Given a list of evaluation task directories
       find all pickle files and return that as
       a list as well as a list of those that failed
    """
    pickle_list = []
    no_pickle_list = []

    if path_list is None:
        logger.error('Path list is None')
        return pickle_list, no_pickle_list

    logger.info('Given ' + str(len(path_list)) + ' paths to look for'
                                                 ' pickle files in')

    for all_stage_7 in path_list:
        full_path = os.path.join(all_stage_7, RMSD_PICKLE)
        if os.path.isfile(full_path):
            pickle_list.append(full_path)
        else:
            logging.info("The pickle file " + full_path +
                         "does not exist")
            no_pickle_list.append(full_path)

    return pickle_list, no_pickle_list


def _get_submission_name_from_pickle_path(path, prefix, suffix,
                                          max_submission_name_width=None):
    """Given path to pickle path in evaluation task
       get the submission name which is basically
       the prefix and suffix values lopped off
    """
    dir_name = os.path.dirname(path)
    base_name = os.path.basename(dir_name)
    sub_name_suffix = re.sub('^' + prefix, '', base_name)
    res = re.sub(suffix, '', sub_name_suffix)
    if max_submission_name_width is None or max_submission_name_width <= 3:
        return res
    if len(res) > max_submission_name_width:
        return res[0:max_submission_name_width-2] + '..'
    return res


def generate_overall_csv(evaluation_path, challenge_dir, post_evaluation_path,
                         candidates_type=LMCSS,
                         eval_stage_prefix='',
                         eval_suffix='',
                         submission_name_width=30):

    not_valid_pickle = 0

    all_pickle_files, non_pickle_list = _get_pickle_paths(evaluation_path)

    candidates_report = os.path.join(challenge_dir, CHALL_FINAL_LOG)

    # Doing 2 checks cause older versions had mispelled Succsessfully in
    # final.log
    # output and new ones fixed this.
    mispelled_candidates = check_case_number(candidates_report,
                                             "Succsessfully generate " +
                                             "this protein:" + candidates_type)
    correctspelled_candidates = check_case_number(candidates_report,
                                                  "Successfully generate "
                                                  "this protein:" +
                                                  candidates_type)
    total_candidates = 0
    if mispelled_candidates > 0:
        total_candidates += mispelled_candidates
    if correctspelled_candidates > 0:
        total_candidates += correctspelled_candidates

    overall_csv = open(os.path.join(post_evaluation_path,
                                    OVERALL_RMSD_PREFIX + candidates_type +
                                    CSV_SUFFIX), 'w')

    summary_txt = open(os.path.join(post_evaluation_path,
                                    SUMMARY_TXT), 'a')

    s_line_format = ('%-' + str(submission_name_width+1) +
                     's%-10s%-10s%-10s%-10s%-12s'
                     '%-4s%-4s%-4s%-4s%-4s%-4s%-4s%-4s\n')
    headerline = (s_line_format % (candidates_type + ' (' +
                                   str(total_candidates) +
                                   ' dockable)',
                                   '#Docked',
                                   'Min RMSD',
                                   'Max RMSD',
                                   'Mean RMSD',
                                   'Median RMSD',
                                   '0<1',
                                   '1<2',
                                   '2<3',
                                   '3<4',
                                   '4<5',
                                   '5<6',
                                   '6<7',
                                   '7+'))
    summary_txt.write(headerline)

    summary_txt.write('-'*int(len(headerline)-1) + '\n')

    data_line_format = '%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\n'
    full_data_lines = [data_line_format % ("SubmissionID for " +
                                           candidates_type,
                                           "# Docked",
                                           "# Dockable",
                                           "Min RMSD",
                                           "Max RMSD",
                                           "Mean RMSD",
                                           "Median RMSD",
                                           "0<1",
                                           "1<2",
                                           "2<3",
                                           "3<4",
                                           "4<5",
                                           "5<6",
                                           "6<7",
                                           "7+")]
    for p_file in all_pickle_files:
        sub_name = _get_submission_name_from_pickle_path(p_file,
                                                         eval_stage_prefix,
                                                         eval_suffix)
        tsub_name = _get_submission_name_from_pickle_path(p_file,
                                                          eval_stage_prefix,
                                                          eval_suffix,
                                                          max_submission_name_width=submission_name_width)
        try:
            d_scores = get_dock_scores_as_list(p_file,
                                               ctype=candidates_type)
            (num_docked, minrmsd, maxrmsd,
             avgrmsd, medianrmsd) = get_list_of_stats(d_scores)

            histo = get_histogram_of_dock_scores(d_scores, BINSIZE,
                                                 NUMBER_BINS)
            if histo is None:
                histo = [0] * NUMBER_BINS

            pc_docked = '(NA%)'
            if total_candidates > 0:
                percent_docked = float(num_docked) / float(total_candidates)
                percent_docked *= 100.0
                pc_docked = ' (%3.0f%%)' % percent_docked

            full_data_lines.append(data_line_format % (sub_name,
                                                       num_docked,
                                                       total_candidates,
                                                       '%.2f' % minrmsd,
                                                       '%.2f' % maxrmsd,
                                                       '%.2f' % avgrmsd,
                                                       '%.2f' % medianrmsd,
                                                       '%d' % histo[0],
                                                       '%d' % histo[1],
                                                       '%d' % histo[2],
                                                       '%d' % histo[3],
                                                       '%d' % histo[4],
                                                       '%d' % histo[5],
                                                       '%d' % histo[6],
                                                       '%d' % histo[7]))
            summary_txt.write(s_line_format % (tsub_name,
                                               str(num_docked) + pc_docked,
                                               '%.2f' % minrmsd,
                                               '%.2f' % maxrmsd,
                                               '%.2f' % avgrmsd,
                                               '%.2f' % medianrmsd,
                                               '%d' % histo[0],
                                               '%d' % histo[1],
                                               '%d' % histo[2],
                                               '%d' % histo[3],
                                               '%d' % histo[4],
                                               '%d' % histo[5],
                                               '%d' % histo[6],
                                               '%d' % histo[7]))
        except IOError:
            logging.exception('Error writing entry')
        except (ValueError, TypeError):
            full_data_lines.append(data_line_format % (sub_name,
                                                       "N/A",
                                                       total_candidates,
                                                       "N/A",
                                                       "N/A",
                                                       "N/A",
                                                       "N/A",
                                                       "N/A",
                                                       "N/A",
                                                       "N/A",
                                                       "N/A",
                                                       "N/A",
                                                       "N/A",
                                                       "N/A",
                                                       "N/A"))
            summary_txt.write(s_line_format % (tsub_name,
                                               "N/A",
                                               "N/A",
                                               "N/A",
                                               "N/A",
                                               "N/A",
                                               "N/A",
                                               "N/A",
                                               "N/A",
                                               "N/A",
                                               "N/A",
                                               "N/A",
                                               "N/A",
                                               "N/A"))
            logging.exception("This pickle file :%s do not have valid "
                              "data for "
                              "candidates type  %s" % (p_file,
                                                       candidates_type))
            not_valid_pickle += 1

    for np_file in non_pickle_list:
        sub_name = _get_submission_name_from_pickle_path(np_file,
                                                         eval_stage_prefix,
                                                         eval_suffix)
        tsub_name = _get_submission_name_from_pickle_path(np_file,
                                                          eval_stage_prefix,
                                                          eval_suffix,
                                                          max_submission_name_width=submission_name_width)
        full_data_lines.append(data_line_format % (sub_name,
                                                   "N/A",
                                                   total_candidates,
                                                   "N/A",
                                                   "N/A",
                                                   "N/A",
                                                   "N/A",
                                                   "N/A",
                                                   "N/A",
                                                   "N/A",
                                                   "N/A",
                                                   "N/A",
                                                   "N/A",
                                                   "N/A",
                                                   "N/A"))
        summary_txt.write(s_line_format % (tsub_name,
                                           "N/A",
                                           "N/A",
                                           "N/A",
                                           "N/A",
                                           "N/A",
                                           "N/A",
                                           "N/A",
                                           "N/A",
                                           "N/A",
                                           "N/A",
                                           "N/A",
                                           "N/A",
                                           "N/A"))

    logging.info("We got : %s cases without pickle files and %s cases have "
                 "invalid pickle files for "
                 "candidates type %s" % (str(len(non_pickle_list)),
                                         not_valid_pickle,
                                         candidates_type))
    overall_csv.writelines(full_data_lines)
    overall_csv.close()

    summary_txt.write('\n\n')
    summary_txt.close()


def main(args):
    """Main entry point of script"""
    desc = """
              Summarizes results for all docking evaluations
              passed into script
    """
    help_formatter = argparse.RawDescriptionHelpFormatter
    parser = argparse.ArgumentParser(description=desc,
                                     formatter_class=help_formatter)
    parser.add_argument("outdir",
                        help="PATH where we will run the evaluate stage")
    parser.add_argument("-e", "--evaluationdir", metavar="PATH",
                        action='append',
                        help="PATH where we could find the evaluation "
                             "stage output")
    parser.add_argument("-c", "--challengedir", metavar="PATH",
                        help="PATH to celpp week directory under challenge "
                             "task ")
    parser.add_argument("--stageprefix", default='stage.7.',
                        help="Evaluation task dir prefix (default stage.7.) ")
    parser.add_argument("--histogrambinsize", default=1,
                        help='Sets histogram bin size in angstroms '
                             '(default 1)')
    parser.add_argument("--histogrambincount", default=3)
    parser.add_argument("--evaluationsuffix",
                        default='.extsubmission.evaluation$|.evaluation$',
                        help="Evaluation task dir suffix regex"
                             "(default .extsubmission.evaluati"
                             "on$|.evaluation$)")

    opt = parser.parse_args(args[1:])
    logging.basicConfig(format='%(asctime)s: %(message)s',
                        datefmt='%m/%d/%y %I:%M:%S',
                        filename=os.path.join(opt.outdir, FINAL_LOG),
                        filemode='w', level=logging.INFO)
    # main calculation
    for candidate_type in [LMCSS, SMCSS, HI_TANIMOTO,
                           HI_RESAPO, HI_RESHOLO]:
        try:
            generate_overall_csv(opt.evaluationdir,
                                 opt.challengedir,
                                 opt.outdir,
                                 candidates_type=candidate_type,
                                 eval_stage_prefix=opt.stageprefix,
                                 eval_suffix=opt.evaluationsuffix)
        except Exception:
            logger.exception('Caught exception running generate_overall_csv '
                             'for candidate type ' + candidate_type)
            continue
    return 0


if __name__ == '__main__':  # pragma: no cover
    sys.exit(main(sys.argv))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_molfilevalidator
----------------------------------

Tests for `molfilevalidator` module.
"""
import sys
import tempfile
import os
import os.path
import shutil
import tarfile

if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest

try:
    OPENEYE_MOD_LOADED = False
    from openeye import oechem
    try:
        if oechem.OEChemIsLicensed():
            OPENEYE_MOD_LOADED = True
        else:
            sys.stderr.write('WARNING: No valid license found for openeye ' +
                             'Please verify OE_LICENSE environment variable'
                             'is set to valid license file\n')
    except AttributeError as ae:
        sys.stderr.write('WARNING Unable to check if openey is licensed' +
                         str(ae))
except ImportError as e:
    pass

from d3r.celpp.task import D3RParameters  # noqa: E402
from d3r import molfilevalidator  # noqa: E402
from d3r.molfilevalidator import D3RAtom  # noqa: E402
from d3r.molfilevalidator import D3RMolecule  # noqa: E402
from d3r.molfilevalidator import D3RMoleculeFromSmileViaOpeneyeFactory \
    # noqa: E402
from d3r.molfilevalidator import D3RMoleculeFromMolFileViaOpeneyeFactory \
    # noqa: E402
from d3r.molfilevalidator import ValidationReport  # noqa: E402
from d3r.molfilevalidator import CompareMolecules  # noqa: E402


class TestMolFileValidator(unittest.TestCase):
    """Tests molfilevalidator.py command line script
    """
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def _get_mol_as_str(self):
        return """
-OEChem-09291710502D

  2  1  0     0  0  0  0  0  0999 V2000
    0.0000    0.0000    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
    1.0000    0.0000    0.0000 N   0  0  0  0  0  0  0  0  0  0  0  0
  1  2  1  0  0  0  0
M  END
"""

    def _get_mol2_as_str(self):
        return """
-OEChem-09291710502D

  2  1  0     0  0  0  0  0  0999 V2000
    0.0000    0.0000    0.0000 N   0  0  0  0  0  0  0  0  0  0  0  0
    1.0000    0.0000    0.0000 N   0  0  0  0  0  0  0  0  0  0  0  0
  1  2  1  0  0  0  0
M  END
"""

    def test_parse_arguments(self):

        gen_mode = molfilevalidator.GENMOLECULEDB_MODE
        params = molfilevalidator._parse_arguments('hi', [gen_mode])

        self.assertEqual(params.mode, gen_mode)
        self.assertEqual(params.moldir, None)
        self.assertEqual(params.skipligand, None)
        self.assertEqual(params.outputfile, None)
        self.assertEqual(params.moleculedb, None)
        self.assertEqual(params.molcsvligandcol, 0)
        self.assertEqual(params.molcsvsmilecol, 1)
        self.assertEqual(params.loglevel, molfilevalidator.DEFAULT_LOG_LEVEL)
        self.assertEqual(params.excludedir, 'SuppInfo')

        params = molfilevalidator._parse_arguments('hi',
                                                   [gen_mode,
                                                    '--moldir', '/foo',
                                                    '--outputfile', 'out',
                                                    '--skipligand', 'FXR_33',
                                                    '--moleculedb', 'mdb',
                                                    '--log', 'DEBUG',
                                                    '--excludedir', 'hi,how'])
        self.assertEqual(params.mode, gen_mode)
        self.assertEqual(params.moldir, '/foo')
        self.assertEqual(params.skipligand, 'FXR_33')
        self.assertEqual(params.outputfile, 'out')
        self.assertEqual(params.moleculedb, 'mdb')
        self.assertEqual(params.loglevel, 'DEBUG')
        self.assertEqual(params.excludedir, 'hi,how')

        val_mode = molfilevalidator.VALIDATE_MODE
        params = molfilevalidator._parse_arguments('hi', [val_mode])
        self.assertEqual(params.mode, val_mode)

    def test_d3ratom(self):
        atom = D3RAtom()
        self.assertEqual(atom.is_hydrogen(), False)
        self.assertEqual(atom.get_atomic_number(), 0)
        self.assertEqual(atom.get_atomic_name(), '')

        atom.set_is_hydrogen(True)
        atom.set_atomic_number(3)
        atom.set_atomic_name('Lithium')

        self.assertEqual(atom.is_hydrogen(), True)
        self.assertEqual(atom.get_atomic_number(), 3)
        self.assertEqual(atom.get_atomic_name(), 'Lithium')

        atom.set_is_hydrogen(False)
        self.assertEqual(atom.is_hydrogen(), False)

    def test_d3rmolecule(self):
        mol = D3RMolecule()
        self.assertEqual(mol.get_atoms(), None)
        mol.set_atoms(['hi', 'bye'])
        self.assertEqual(mol.get_atoms(), ['hi', 'bye'])

    @unittest.skipIf(OPENEYE_MOD_LOADED is False,
                     'No valid openeye installation or license found')
    def test_d3rmoleculefrommolfileviaopeneyefactory(self):
        mol = self._get_mol_as_str()
        temp_dir = tempfile.mkdtemp()
        try:
            factory = D3RMoleculeFromMolFileViaOpeneyeFactory()
            badmol = os.path.join(temp_dir, 'bad.mol')

            with open(badmol, 'w') as f:
                f.write('hi\n')
                f.flush()
                f.close()
            try:
                factory.get_d3rmolecule(badmol)
                self.fail('Expected ValueError')
            except ValueError as e:
                self.assertEqual(str(e), 'OEReadMolecule returned False when '
                                         'trying to read mol file')
            mfile = os.path.join(temp_dir, 'foo.mol')
            with open(mfile, 'w') as f:
                f.write(mol)
                f.flush()
                f.close()
            d3rmol = factory.get_d3rmolecule(mfile)
            self.assertEqual(len(d3rmol.get_atoms()), 2)
            self.assertEqual(d3rmol.get_atoms()[0].is_hydrogen(), False)
            self.assertEqual(d3rmol.get_atoms()[0].get_atomic_number(), 6)
            self.assertEqual(d3rmol.get_atoms()[1].is_hydrogen(), False)
            self.assertEqual(d3rmol.get_atoms()[1].get_atomic_number(), 7)
        finally:
            shutil.rmtree(temp_dir)

    @unittest.skipIf(OPENEYE_MOD_LOADED is False,
                     'No valid openeye installation or license found')
    def test_d3rmoleculefromsmileviaopeneyefactory(self):
        factory = D3RMoleculeFromSmileViaOpeneyeFactory()
        d3rmol = factory.get_d3rmolecule('CN')
        self.assertEqual(len(d3rmol.get_atoms()), 2)
        self.assertEqual(d3rmol.get_atoms()[0].is_hydrogen(), False)
        self.assertEqual(d3rmol.get_atoms()[0].get_atomic_number(), 6)
        self.assertEqual(d3rmol.get_atoms()[1].is_hydrogen(), False)
        self.assertEqual(d3rmol.get_atoms()[1].get_atomic_number(), 7)

    def test_get_molecule_weight_and_summary(self):
        # test where molecule is None
        (ha, mw, adic, smi_str) = molfilevalidator.\
            get_molecule_weight_and_summary(None)
        self.assertEqual(ha, -1)
        self.assertEqual(mw, -1)
        self.assertEqual(adic, {})
        self.assertEqual(smi_str, '')
        mol = D3RMolecule()

        # test None for atoms
        (ha, mw, adic, smi_str) = molfilevalidator.\
            get_molecule_weight_and_summary(mol)
        self.assertEqual(ha, 0)
        self.assertEqual(mw, 0)
        self.assertEqual(adic, {})
        self.assertEqual(smi_str, '')

        # test with empty list for atoms
        mol.set_atoms([])
        (ha, mw, adic, smi_str) = molfilevalidator.\
            get_molecule_weight_and_summary(mol)
        self.assertEqual(ha, 0)
        self.assertEqual(mw, 0)
        self.assertEqual(adic, {})
        self.assertEqual(smi_str, None)

        # test with 1 atom is hydrogen
        atom = D3RAtom()
        atom.set_is_hydrogen(True)
        atom.set_atomic_number(1)
        mol.set_atoms([atom])
        mol.set_canonical_smiles_str('CC')
        (ha, mw, adic, smi_str) = molfilevalidator.\
            get_molecule_weight_and_summary(mol)
        self.assertEqual(ha, 0)
        self.assertEqual(mw, 0)
        self.assertEqual(adic, {})
        self.assertEqual(smi_str, 'CC')

        # test with 1 atom non hydrogen
        atom.set_is_hydrogen(False)
        atom.set_atomic_number(5)
        mol.set_atoms([atom])
        mol.set_canonical_smiles_str('CCN')
        (ha, mw, adic, smi_str) = molfilevalidator.\
            get_molecule_weight_and_summary(mol)
        self.assertEqual(ha, 1)
        self.assertEqual(mw, 5)
        self.assertEqual(adic, {5: 1})
        self.assertEqual(smi_str, 'CCN')

        # test with 2 atoms where one is 1 non hydrogen
        hatom = D3RAtom()
        hatom.set_is_hydrogen(True)
        hatom.set_atomic_number(1)

        mol.set_atoms([atom, hatom])
        (ha, mw, adic, smi_str) = molfilevalidator.\
            get_molecule_weight_and_summary(mol)
        self.assertEqual(ha, 1)
        self.assertEqual(mw, 5)
        self.assertEqual(adic, {5: 1})
        self.assertEqual(smi_str, 'CCN')

        # test with 4 atoms where one is hydrogen
        catom = D3RAtom()
        catom.set_atomic_number(6)
        catom.set_is_hydrogen(False)
        mol.set_atoms([atom, hatom, catom, catom])
        (ha, mw, adic, smi_str) = molfilevalidator.\
            get_molecule_weight_and_summary(mol)
        self.assertEqual(ha, 3)
        self.assertEqual(mw, 17)
        self.assertEqual(adic, {5: 1, 6: 2})
        self.assertEqual(smi_str, 'CCN')

    def test_validation_report(self):
        vr = ValidationReport()

        # test on empty
        self.assertEqual(vr.get_as_string(), '')

        # add a ligand error with none for all fields
        vr.add_ligand_error(None, None, None)
        self.assertTrue('\nLigand Errors\n'
                        '------------\n\n' in vr.get_as_string())
        self.assertTrue('\n\nIn file: None ligand: '
                        'None\n\tNone' in vr.get_as_string())

        # add a molecule with no tuples
        vr = ValidationReport()
        vr.add_molecule_error('x.mol', 'DDD_3', None, None, 'yo')
        self.assertTrue('\nMolecule Errors\n' in vr.get_as_string())
        self.assertTrue('\n\nIn file: x.mol ligand: '
                        'DDD_3\n\tyo\n\n' in vr.get_as_string())

        vr = ValidationReport()
        vr.add_ligand_error('/foo/my.mol', 'ABC_1', 'some error')
        self.assertTrue('\nLigand Errors\n'
                        '------------\n\n' in vr.get_as_string())
        self.assertTrue('\n\nIn file: my.mol ligand: '
                        'ABC_1\n\tsome error' in vr.get_as_string())

        # add a molecule error
        vr.add_molecule_error('/blah/b.mol', 'XXX_2', (1, 2, {6: 2}, 'CC'),
                              (3, 4, {6: 5}, 'CC'), 'ha')
        self.assertTrue('\nLigand Errors\n'
                        '------------\n\n' in vr.get_as_string())
        self.assertTrue('\n\nIn file: my.mol ligand: '
                        'ABC_1\n\tsome error' in vr.get_as_string())
        self.assertTrue('\nMolecule Errors\n' in vr.get_as_string())

        self.assertTrue('\n\nIn file: b.mol ligand: '
                        'XXX_2 ha' in vr.get_as_string())
        self.assertTrue('\n\tExpected 3 non hydrogen atoms, '
                        'but got 1' in vr.get_as_string())
        self.assertTrue('\n\tExpected 4 for non hydrogen atomic weight, '
                        'but got 2' in vr.get_as_string())
        self.assertTrue('\n\tExpected atom map { atomic #: # atoms,...} '
                        '{6: 5}, but got {6: 2}\n\n' in vr.get_as_string())

    def test_compare_molecules(self):
        cm = CompareMolecules({})
        vr = ValidationReport()

        # test with molecule not in db
        res = cm.compare_molecules('m.mol', vr, 'XXX_1', D3RMolecule())
        self.assertEqual(res, False)
        self.assertEqual(len(vr.get_molecule_errors()), 0)
        ligand_errors = vr.get_ligand_errors()
        self.assertEqual(len(ligand_errors), 1)
        self.assertEqual(ligand_errors[0][ValidationReport.MOLFILE], 'm.mol')
        self.assertEqual(ligand_errors[0][ValidationReport.LIGAND], 'XXX_1')
        self.assertEqual(ligand_errors[0][ValidationReport.MESSAGE],
                         'ligand not in molecule database')

        # test with molecule not matching number heavy atoms
        mol = D3RMolecule()
        atom = D3RAtom()
        atom.set_is_hydrogen(False)
        atom.set_atomic_number(5)
        mol.set_atoms([atom])
        mol.set_canonical_smiles_str('CC')

        vr = ValidationReport()
        cm = CompareMolecules({'XXX_1': (3, 5, {5: 1}, 'CC')})
        res = cm.compare_molecules('m.mol', vr, 'XXX_1', mol)
        self.assertEqual(res, False)
        m_errors = vr.get_molecule_errors()
        self.assertEqual(len(m_errors), 1)
        self.assertEqual(m_errors[0][ValidationReport.MOLFILE], 'm.mol')
        self.assertEqual(m_errors[0][ValidationReport.LIGAND], 'XXX_1')
        self.assertEqual(m_errors[0][ValidationReport.USERMOL],
                         (1, 5, {5: 1}, 'CC'))
        self.assertEqual(m_errors[0][ValidationReport.EXPECTEDMOL],
                         (3, 5, {5: 1}, 'CC'))
        self.assertTrue('Number of heavy atoms '
                        'and or' in m_errors[0][ValidationReport.MESSAGE])

        # test with molecule not matching molecular weight
        vr = ValidationReport()
        cm = CompareMolecules({'XXX_1': (1, 6, {5: 1}, 'CC')})
        res = cm.compare_molecules('m.mol', vr, 'XXX_1', mol)
        self.assertEqual(res, False)
        m_errors = vr.get_molecule_errors()
        self.assertEqual(len(m_errors), 1)
        self.assertEqual(m_errors[0][ValidationReport.MOLFILE], 'm.mol')
        self.assertEqual(m_errors[0][ValidationReport.LIGAND], 'XXX_1')
        self.assertEqual(m_errors[0][ValidationReport.USERMOL],
                         (1, 5, {5: 1}, 'CC'))
        self.assertEqual(m_errors[0][ValidationReport.EXPECTEDMOL],
                         (1, 6, {5: 1}, 'CC'))
        self.assertTrue('Number of heavy atoms '
                        'and or' in m_errors[0][ValidationReport.MESSAGE])

        # test with matching molecule but SMILE String differs
        vr = ValidationReport()
        cm = CompareMolecules({'XXX_1': (1, 5, {5: 1}, 'CN')})
        res = cm.compare_molecules('m.mol', vr, 'XXX_1', mol)
        self.assertEqual(res, False)
        m_errors = vr.get_molecule_errors()
        self.assertEqual(len(m_errors), 1)
        self.assertEqual(m_errors[0][ValidationReport.MOLFILE], 'm.mol')
        self.assertEqual(m_errors[0][ValidationReport.LIGAND], 'XXX_1')
        self.assertEqual(m_errors[0][ValidationReport.USERMOL],
                         (1, 5, {5: 1}, 'CC'))
        self.assertEqual(m_errors[0][ValidationReport.EXPECTEDMOL],
                         (1, 5, {5: 1}, 'CN'))
        self.assertTrue('Canonical SMILE strings do NOT '
                        'match' in m_errors[0][ValidationReport.MESSAGE])

        # test with matching molecule but SMILE String differs
        # but this time we are skipping smile string comparison
        vr = ValidationReport()
        cm = CompareMolecules({'XXX_1': (1, 5, {5: 1}, 'CN')},
                              skipsmilecompare=True)
        res = cm.compare_molecules('m.mol', vr, 'XXX_1', mol)
        self.assertEqual(res, True)
        m_errors = vr.get_molecule_errors()
        self.assertEqual(len(m_errors), 0)

        # test with full match
        vr = ValidationReport()
        cm = CompareMolecules({'XXX_1': (1, 5, {5: 1}, 'CC')})
        res = cm.compare_molecules('m.mol', vr, 'XXX_1', mol)
        self.assertEqual(res, True)
        m_errors = vr.get_molecule_errors()
        self.assertEqual(len(m_errors), 0)

    def test_get_ligand_name_from_file_name(self):
        try:
            molfilevalidator._get_ligand_name_from_file_name(None)
            self.fail('Expected ValueError')
        except ValueError as e:
            self.assertEqual(str(e), 'file_name cannot be None')

        try:
            molfilevalidator._get_ligand_name_from_file_name('hi')
            self.fail('Expected ValueError')
        except ValueError as e:
            self.assertEqual(str(e), 'Error parsing ligand name from file '
                                     'name: hi')

        res = molfilevalidator._get_ligand_name_from_file_name('DSV-'
                                                               'FXR_12-1.mol')
        self.assertEqual(res, 'FXR_12')

    def test_generate_molecule_database_fromcsv_molcsv_is_none(self):
        params = D3RParameters()
        params.molcsv = None
        res = molfilevalidator.\
            _generate_molecule_database_fromcsv(params, None)
        self.assertEqual(res, 1)

    def test_generate_molecule_database_no_moldir_is_none(self):
        params = D3RParameters()
        params.moldir = None
        res = molfilevalidator.\
            _generate_molecule_database_frommolfiles(params, None)
        self.assertEqual(res, 1)

    @unittest.skipIf(OPENEYE_MOD_LOADED is False,
                     'No valid openeye installation or license found')
    def test_generate_molecule_database_from_csv_with_validcsv(self):
        temp_dir = tempfile.mkdtemp()
        try:
            csvfile = os.path.join(temp_dir, 'foo.csv')
            with open(csvfile, 'w') as f:
                f.write('Ligand_ID, Smiles, Target, Subchallenge\n')
                f.write('ABC_1,FC(F),abc,something\n')
                f.write('DEF_1,CC,abc,something\n')
                f.flush()
            params = D3RParameters()
            params.molcsv = csvfile
            params.outputfile = os.path.join(temp_dir, 'res.pickle')
            params.molcsvligandcol = 0
            params.molcsvsmilecol = 1
            factory = D3RMoleculeFromSmileViaOpeneyeFactory()
            res = molfilevalidator._generate_molecule_database_fromcsv(params,
                                                                       factory)
            self.assertEqual(res, 0)
            self.assertTrue(os.path.isfile(params.outputfile))

            params.moleculedb = params.outputfile
            ligand_dic = molfilevalidator._get_molecule_database(params)
            self.assertEqual(len(ligand_dic), 2)
            self.assertEqual(ligand_dic['ABC_1'][0], 3)
            self.assertEqual(ligand_dic['DEF_1'][0], 2)
        finally:
            shutil.rmtree(temp_dir)

    def test_generate_molecule_database_no_molfiles(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.moldir = temp_dir
            output = os.path.join(temp_dir, 'foo.pickle')
            params.outputfile = output
            res = molfilevalidator.\
                _generate_molecule_database_frommolfiles(params, None)
            self.assertEqual(res, 4)
            self.assertFalse(os.path.isfile(output))
        finally:
            shutil.rmtree(temp_dir)

    @unittest.skipIf(OPENEYE_MOD_LOADED is False,
                     'No valid openeye installation or license found')
    def test_generate_molecule_database_one_molfile_with_openeye(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.moldir = temp_dir
            output = os.path.join(temp_dir, 'foo.pickle')
            params.outputfile = output
            mymol = os.path.join(temp_dir, 'my-ABC_1-1.mol')
            with open(mymol, 'w') as f:
                f.write(self._get_mol_as_str())
                f.flush()
            fac = D3RMoleculeFromMolFileViaOpeneyeFactory()
            res = molfilevalidator.\
                _generate_molecule_database_frommolfiles(params, fac)
            self.assertEqual(res, 0)
            self.assertTrue(os.path.isfile(output))

            params.moleculedb = output
            moldb = molfilevalidator._get_molecule_database(params)
            self.assertEqual(len(moldb), 1)
            self.assertEqual(moldb['ABC_1'][0], 2)
            self.assertEqual(moldb['ABC_1'][1], 13)
        finally:
            shutil.rmtree(temp_dir)

    @unittest.skipIf(OPENEYE_MOD_LOADED is False,
                     'No valid openeye installation or license found')
    def test_generate_molecule_database_two_molfiles_with_openeye(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.moldir = temp_dir
            output = os.path.join(temp_dir, 'foo.pickle')
            params.outputfile = output
            mymol = os.path.join(temp_dir, 'my-ABC_1-1.mol')
            with open(mymol, 'w') as f:
                f.write(self._get_mol_as_str())
                f.flush()

            mymol2 = os.path.join(temp_dir, 'yo-XXX_2-3.mol')
            with open(mymol2, 'w') as f:
                f.write(self._get_mol2_as_str())
                f.flush()

            fac = D3RMoleculeFromMolFileViaOpeneyeFactory()
            res = molfilevalidator.\
                _generate_molecule_database_frommolfiles(params, fac)
            self.assertEqual(res, 0)
            self.assertTrue(os.path.isfile(output))

            params.moleculedb = output
            moldb = molfilevalidator._get_molecule_database(params)
            self.assertEqual(len(moldb), 2)
            self.assertEqual(moldb['ABC_1'][0], 2)
            self.assertEqual(moldb['ABC_1'][1], 13)
            self.assertEqual(moldb['XXX_2'][0], 2)
            self.assertEqual(moldb['XXX_2'][1], 14)
        finally:
            shutil.rmtree(temp_dir)

    def test_molfile_from_tarfile_generator_no_tarfile(self):
        temp_dir = tempfile.mkdtemp()
        try:
            n = os.path.join(temp_dir, 'foo.tar.gz')
            try:
                list(molfilevalidator._molfile_from_tarfile_generator(n))
                self.fail('Expected IOError')
            except IOError as e:
                self.assertTrue('No such file' in str(e))

        finally:
            shutil.rmtree(temp_dir)

    def test_molfile_from_tarfile_generator_no_mols_in_tar(self):
        temp_dir = tempfile.mkdtemp()
        try:
            afile = os.path.join(temp_dir, 'foo.pdb')
            with open(afile, 'w') as f:
                f.write(self._get_mol_as_str())
                f.flush()
            myf = os.path.join(temp_dir, 'foo.tar.gz')
            tf = tarfile.open(myf, 'w:gz')
            tf.add(afile)
            tf.close()

            mols = list(molfilevalidator._molfile_from_tarfile_generator(myf))
            self.assertEqual(len(mols), 0)

            mols = list(molfilevalidator.
                        _molfile_from_tarfile_generator(myf,
                                                        direxclude='blah'))
            self.assertEqual(len(mols), 0)

        finally:
            shutil.rmtree(temp_dir)

    def test_molfile_from_tarfile_generator_one_mol_in_tar(self):
        temp_dir = tempfile.mkdtemp()
        try:

            afile = os.path.join(temp_dir, 'foo.pdb')
            with open(afile, 'w') as f:
                f.write(self._get_mol_as_str())
                f.flush()

            mfile = os.path.join(temp_dir, 'yo.mol')
            with open(mfile, 'w') as f:
                f.write(self._get_mol_as_str())
                f.flush()

            myf = os.path.join(temp_dir, 'foo.tar.gz')
            tf = tarfile.open(myf, 'w:gz')
            tf.add(afile)
            tf.add(mfile, arcname='SuppInfo/hi.mol')
            tf.close()

            mols = list(molfilevalidator._molfile_from_tarfile_generator(myf))
            self.assertEqual(len(mols), 1)
            self.assertTrue(mols[0].endswith('hi.mol'))

            # try with exclude enabled not match
            mols = list(molfilevalidator.
                        _molfile_from_tarfile_generator(myf,
                                                        direxclude='Supp,yo'))
            self.assertEqual(len(mols), 1)

            # try with exclude enabled
            mols = list(molfilevalidator.
                        _molfile_from_tarfile_generator(myf,
                                                        direxclude='SuppInfo'))
            self.assertEqual(len(mols), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_molfile_from_tarfile_generator_two_mol_in_tar(self):
        temp_dir = tempfile.mkdtemp()
        try:
            afile = os.path.join(temp_dir, 'foo.pdb')
            with open(afile, 'w') as f:
                f.write(self._get_mol_as_str())
                f.flush()

            mfile = os.path.join(temp_dir, 'yo.mol')
            with open(mfile, 'w') as f:
                f.write(self._get_mol_as_str())
                f.flush()

            m2file = os.path.join(temp_dir, 'xxd.mol')
            with open(m2file, 'w') as f:
                f.write(self._get_mol_as_str())
                f.flush()

            myf = os.path.join(temp_dir, 'foo.tar.gz')
            tf = tarfile.open(myf, 'w:gz')
            tf.add(afile)
            tf.add(mfile, arcname='hi.mol')
            tf.add(m2file, arcname='bye.mol')
            tf.close()

            mols = list(molfilevalidator._molfile_from_tarfile_generator(myf))
            self.assertEqual(len(mols), 2)
            self.assertTrue(mols[0].endswith('hi.mol'))
            self.assertTrue(mols[1].endswith('bye.mol'))
        finally:
            shutil.rmtree(temp_dir)

    def test_validate_molfiles_in_tarball_usersubmission_is_none(self):
        params = D3RParameters()
        params.usersubmission = None
        res = molfilevalidator._validate_molfiles_in_tarball(params, None,
                                                             None)
        self.assertEqual(res, 1)

    @unittest.skipIf(OPENEYE_MOD_LOADED is False,
                     'No valid openeye installation or license found')
    def test_validate_molfiles_in_tarball_real(self):
        temp_dir = tempfile.mkdtemp()
        try:

            mfile = os.path.join(temp_dir, 'yo.mol')
            with open(mfile, 'w') as f:
                f.write(self._get_mol_as_str())
                f.flush()
            myf = os.path.join(temp_dir, 'foo.tar.gz')

            bfile = os.path.join(temp_dir, 'x.mol')
            with open(bfile, 'w') as f:
                f.write('blah\n')
                f.flush()
                f.close()
            tf = tarfile.open(myf, 'w:gz')
            tf.add(mfile, arcname='yo.mol')
            tf.add(mfile, arcname='x-ABC_1-1.mol')
            tf.add(bfile, arcname='z-BBB_2-213.mol')
            tf.close()

            params = D3RParameters()
            params.usersubmission = myf
            params.excludedir = 'SuppInfo'
            moldb = {}
            moldb['ABC_1'] = (1, 2, {7: 2}, 'CC')
            molfactory = D3RMoleculeFromMolFileViaOpeneyeFactory()
            res = molfilevalidator._validate_molfiles_in_tarball(params,
                                                                 molfactory,
                                                                 moldb)
            self.assertTrue('parsing ligand name from file '
                            'name: yo.mol' in res.get_as_string())
            self.assertTrue('1 non hydrogen atoms, but '
                            'got 2' in res.get_as_string())
            self.assertTrue('2 for non hydrogen atomic weight, but '
                            'got 13' in res.get_as_string())
            self.assertTrue('BBB_2\n\tligand not in molecule '
                            in res.get_as_string())
        finally:
            shutil.rmtree(temp_dir)

    @unittest.skipIf(OPENEYE_MOD_LOADED is False,
                     'No valid openeye installation or license found')
    def test_validate_molfiles_in_tarball_real_with_skiplist(self):
        temp_dir = tempfile.mkdtemp()
        try:
            myf = os.path.join(temp_dir, 'foo.tar.gz')
            mfile = os.path.join(temp_dir, 'yo.mol')
            with open(mfile, 'w') as f:
                f.write(self._get_mol_as_str())
                f.flush()
            with tarfile.open(myf, 'w:gz') as tf:

                tf.add(mfile, arcname='x-ABC_1-1.mol')

            params = D3RParameters()
            params.usersubmission = myf
            params.excludedir = 'SuppInfo'
            params.skipligand = 'XXX_3,ABC_1'
            moldb = {}
            moldb['ABC_1'] = (1, 2, {7: 2})
            molfactory = D3RMoleculeFromMolFileViaOpeneyeFactory()
            res = molfilevalidator._validate_molfiles_in_tarball(params,
                                                                 molfactory,
                                                                 moldb)
            self.assertEqual(res.get_as_string(), '')
        finally:
            shutil.rmtree(temp_dir)

    def test_main_with_error(self):
        res = molfilevalidator.main(['yo', molfilevalidator.VALIDATE_MODE])
        self.assertEqual(res, 2)

        res = molfilevalidator.main(['yo',
                                     molfilevalidator.GENMOLECULEDB_MODE])
        self.assertEqual(res, 3)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'jwagner'


import unittest
import tempfile
import os.path
import shutil
import os
import sys

from d3r.celpp.task import D3RParameters
from d3r.celpp.task import D3RTask
from d3r.celpp.blastnfilter import BlastNFilterTask
from d3r.celpp.challengedata import ChallengeDataTask
from d3r.celpp.evaluation import EvaluationTaskFactory
from d3r.celpp.evaluation import EvaluationTask
from d3r.celpp.participant import ParticipantDatabase
from d3r.celpp.participant import Participant
from d3r.celpp.task import SmtpEmailer
from d3r.celpp.evaluation import EvaluationEmailer
from mock import Mock


class TestChainPermuter(unittest.TestCase):
    def setUp(self):
        try:
            import openeye.oechem  # noqa: F401
            sys.stdout.write("openeye.oechem importable.\n")
            self.has_oechem = True
        except Exception:
            sys.stdout.write("openeye.oechem not importable.\n")
            self.has_oechem = False

        ret_code = os.system('PYTHONPATH= $SCHRODINGER/run '
                             'split_structure.py &> '
                             '/dev/null')
        if ret_code == 512:
            sys.stdout.write("Schrodinger commandline available.\n")
            self.has_schrodinger = True
        else:
            sys.stdout.write("Schrodinger commandline not available.\n")
            self.has_schrodinger = False

    def test_permute_chains_and_take_min_rmsd(self):
        if not(self.has_oechem) or not (self.has_schrodinger):
            sys.stdout.write("Missing essential component. Skipping test.\n")
            return
        test_scale = 'github'
        method_list = ['autodockvina']
        try:
            temp_dir = tempfile.mkdtemp()
            for method in method_list:
                params = D3RParameters()
                params.evaluation = 'evaluate.py'
                params.pdbdb = os.path.abspath('tests/celpp/eval_test_data/'
                                               '%s_test_data/mini_pdb/'
                                               % test_scale)

                # Make blastnfilter task
                blastnfiltertask = BlastNFilterTask(temp_dir, params)
                blastnfiltertask.create_dir()
                open(os.path.join(blastnfiltertask.get_dir(),
                                  D3RTask.COMPLETE_FILE), 'a').close()
                source_dir = os.path.abspath('tests/celpp/eval_test_data/'
                                             '%s_test_data/stage.3.'
                                             'blastnfilter/'
                                             % test_scale)
                os.system('cp -r %s/* %s' % (source_dir,
                                             blastnfiltertask.get_dir()))

                # Make challengedata task
                challengedatatask = ChallengeDataTask(temp_dir, params)
                challengedatatask.create_dir()
                open(os.path.join(challengedatatask.get_dir(),
                                  D3RTask.COMPLETE_FILE), 'a').close()
                source_dir = os.path.abspath('tests/celpp/eval_test_data/'
                                             '%s_test_data/stage.4.'
                                             'challengedata/'
                                             % test_scale)
                os.system('cp -r %s/* %s' % (source_dir,
                                             challengedatatask.get_dir()))

                # Make dock task
                docktask = D3RTask(temp_dir, params)
                docktask.set_name(method)
                docktask.set_stage(EvaluationTaskFactory.DOCKSTAGE)
                docktask.create_dir()
                open(os.path.join(docktask.get_dir(), D3RTask.COMPLETE_FILE),
                     'a').close()

                source_dir = os.path.abspath('tests/celpp/eval_test_data/'
                                             '%s_test_data/stage.6.%s/'
                                             % (test_scale, method))
                os.system('cp -r %s/* %s' % (source_dir, docktask.get_dir()))
                print 'docktask.get_dir()', docktask.get_dir()
                print 'temp_dir', temp_dir
                task = EvaluationTask(temp_dir, '%s.evaluation' % (method),
                                      docktask, params)

                plist = [Participant('1name', '1d3rusername', '12345',
                                     'bob@bob.com,joe@joe.com')]
                smtpemailer = SmtpEmailer()
                mockserver = D3RParameters()
                mockserver.sendmail = Mock()
                mockserver.quit = Mock()
                smtpemailer.set_alternate_smtp_server(mockserver)
                emailer = EvaluationEmailer(ParticipantDatabase(plist), None)
                emailer.set_alternate_smtp_emailer(smtpemailer)
                task.set_evaluation_emailer(emailer)

                task.run()
                val = task.get_evaluation_summary()
                self.assertEqual(val,
                                 '\nEvaluation of docking\n==================='
                                 '==\nTarge'
                                 't_PDBID        LMCSS            SMCSS       '
                                 '     h'
                                 'iResApo         hiResHolo        hiTanimoto '
                                 '      '
                                 'LMCSS_ori_distance \n\nSummary Statistics\n'
                                 '\nNumber_of'
                                 '_cases     0                0               '
                                 ' 1    '
                                 '            0                0              '
                                 '  \nAve'
                                 'rage                                        '
                                 '      '
                                 ' 6.447                                     '
                                 '       '
                                 '  \nMaximum                                '
                                 '        '
                                 '       6.447                               '
                                 '       '
                                 '        \nMinimum                           '
                                 '       '
                                 '             6.447                          '
                                 '      '
                                 '              \nMedian                      '
                                 '       '
                                 '                   6.447                    '
                                 '      '
                                 '                    \n\nIndividual Results\n'
                                 '\n5t6d    '
                                 '                                            '
                                 '  6.44'
                                 '7 (2.176 )                                  '
                                 '      '
                                 '              \n\n')
                self.assertEqual(task.get_error(), None)
        finally:
            # pass
            shutil.rmtree(temp_dir)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_post_evaluation
----------------------------------

Tests for `post_evaluation` module.
"""

import unittest
import tempfile
import os
import os.path
import shutil
import math

from d3r import evaluate
from d3r.evaluate import DataContainer


class TestEvaluate(unittest.TestCase):
    """Tests post_evaluation commandline script
    """
    def setUp(self):
        pass

    def test_clean_up_list_of_value(self):
        # test on None
        res = evaluate.clean_up_list_of_value(None)
        self.assertEqual(res, [])

        # test on empty list
        res = evaluate.clean_up_list_of_value([])
        self.assertEqual(res, [])

        # test on list with only None values
        res = evaluate.clean_up_list_of_value([None, None])
        self.assertEqual(res, [])

        # test on list mixed list
        res = evaluate.clean_up_list_of_value([None, 1, 2, None])
        self.assertEqual(res, [1, 2])

    def test_calculate_average_min_max_median(self):
        # pass in None
        res = evaluate.calculate_average_min_max_median(None)
        spacefield = "%-15s, " % (" ")
        self.assertEqual(res, (spacefield, spacefield, spacefield, spacefield))

        # pass in empty list
        res = evaluate.calculate_average_min_max_median([])
        self.assertEqual(res, (spacefield, spacefield, spacefield, spacefield))

        form = '%-15.3f, '
        # pass in list with None and 1 value
        res = evaluate.calculate_average_min_max_median([None, 5])
        avg = form % 5
        min = form % 5
        max = form % 5
        med = form % 5
        self.assertEqual(res, (avg, min, max, med))

        # pass in list with 2 values
        res = evaluate.calculate_average_min_max_median([3, 5])
        avg = form % 4
        min = form % 3
        max = form % 5
        med = form % 4
        self.assertEqual(res, (avg, min, max, med))

        # pass in list with 3 values
        res = evaluate.calculate_average_min_max_median([3, 5, 9])
        avg = form % 5.666666
        min = form % 3
        max = form % 9
        med = form % 5
        self.assertEqual(res, (avg, min, max, med))

        # pass in list with 4 values
        res = evaluate.calculate_average_min_max_median([8, 3, 5, 9])
        avg = form % 6.25
        min = form % 3
        max = form % 9
        med = form % 6.5
        self.assertEqual(res, (avg, min, max, med))

    def test_data_container_get_all_docked_type(self):
        dc = DataContainer()
        self.assertEqual(dc.get_all_docked_type(docked_type='LMCSS'), [])

        dc.register('6abc', 'LMCSS', 1)
        self.assertEqual(dc.get_all_docked_type(docked_type='LMCSS'), [1])

        dc.register('6abc', 'LMCSS_dis', 2)
        self.assertEqual(dc.get_all_docked_type(docked_type='LMCSS'), [1])

        # verify multiple
        dc.register('7def', 'LMCSS', 3)
        res = dc.get_all_docked_type(docked_type='LMCSS')
        self.assertEqual(len(res), 2)
        self.assertTrue(1 in res)
        self.assertTrue(3 in res)

        # verify insertion of value for already existing docked_type is ignored
        # why this is setup this way is anybody's guess, but the test
        # verifies the code is doing this.
        dc.register('6abc', 'LMCSS_dis', 5)
        self.assertEqual(dc.get_all_docked_type(docked_type='LMCSS_dis'), [2])

    def test_data_container_layout_plain_empty(self):
        temp_dir = tempfile.mkdtemp()
        try:
            dc = DataContainer()
            rmsd = os.path.join(temp_dir, 'rmsd')
            dc.layout_plain(plain_filename=rmsd)

            rmsdcsv = rmsd + '.csv'
            rmsdtxt = rmsd + '.txt'
            self.assertTrue(os.path.isfile(rmsdcsv))
            self.assertTrue(os.path.isfile(rmsdtxt))

            f = open(rmsdcsv, 'r')
            lines = f.readlines()
            f.close()
            self.assertTrue(lines[0].startswith('Target_PDBID'))
            self.assertTrue(lines[2].startswith('Summary Statistics'))
            self.assertTrue(lines[4].startswith('Number_of_cases'))
            self.assertTrue(lines[5].startswith('Average'))
            self.assertTrue(lines[6].startswith('Maximum'))
            self.assertTrue(lines[7].startswith('Minimum'))
            self.assertTrue(lines[8].startswith('Median'))
            self.assertTrue(lines[10].startswith('Individual Results'))

            f = open(rmsdtxt, 'r')
            lines = f.readlines()
            f.close()
            self.assertTrue(lines[0].startswith('Target_PDBID'))
            self.assertTrue(lines[2].startswith('Summary Statistics'))
            self.assertTrue(lines[4].startswith('Number_of_cases'))
            self.assertTrue(lines[5].startswith('Average'))
            self.assertTrue(lines[6].startswith('Maximum'))
            self.assertTrue(lines[7].startswith('Minimum'))
            self.assertTrue(lines[8].startswith('Median'))
            self.assertTrue(lines[10].startswith('Individual Results'))

        finally:
            shutil.rmtree(temp_dir)

    def test_data_container_layout_plain_with_data(self):
        temp_dir = tempfile.mkdtemp()
        try:
            dc = DataContainer()
            dc.register('1abc', 'LMCSS', 10.5)
            dc.register('1abc', 'SMCSS', 1.0)
            dc.register('1abc', 'hiResApo', 2.0)
            dc.register('1abc', 'hiResHolo', 3.0)
            dc.register('1abc', 'hiTanimoto', 4.0)
            dc.register('1abc', 'LMCSS_ori', 5.0)
            dc.register('1abc', 'LMCSS_ori_dis', 6.0)

            dc.register('1abc', 'LMCSS_dis', 1.0)
            dc.register('2abc', 'LMCSS', 20.5)
            dc.register('3abc', 'SMCSS', 2.0)
            dc.register('4abc', 'SMCSS', 3.0)

            rmsd = os.path.join(temp_dir, 'rmsd')
            dc.layout_plain(plain_filename=rmsd)

            rmsdcsv = rmsd + '.csv'
            rmsdtxt = rmsd + '.txt'
            self.assertTrue(os.path.isfile(rmsdcsv))
            self.assertTrue(os.path.isfile(rmsdtxt))

            f = open(rmsdcsv, 'r')
            lines = f.readlines()
            f.close()
            self.assertTrue(lines[0].startswith('Target_PDBID'))
            self.assertTrue(lines[2].startswith('Summary Statistics'))
            self.assertTrue(lines[4].startswith('Number_of_cases'))
            self.assertTrue('2              , 3              , 1'
                            '              , 1              , 1' in lines[4])
            self.assertTrue(lines[5].startswith('Average'))
            self.assertTrue('15.500         , 2.000          , 2.000'
                            '          , 3.000          , 4.000' in lines[5])
            self.assertTrue(lines[6].startswith('Maximum'))
            self.assertTrue('20.500         , 3.000          , 2.000'
                            '          , 3.000          , 4.000' in lines[6])
            self.assertTrue(lines[7].startswith('Minimum'))
            self.assertTrue('10.500         , 1.000          , 2.000'
                            '          , 3.000          , 4.000' in lines[7])
            self.assertTrue(lines[8].startswith('Median'))
            self.assertTrue('15.500         , 2.000          , 2.000'
                            '          , 3.000          , 4.000' in lines[8])
            self.assertTrue(lines[10].startswith('Individual Results'))
            self.assertTrue('4abc,                              , 3.000' in
                            lines[12])
            self.assertTrue('3abc,                              , 2.000 ' in
                            lines[13])
            self.assertTrue('2abc,               20.500' in lines[14])
            self.assertTrue('1abc,               10.500(1.000 ) , 1.000'
                            '          , 2.000          , 3.000'
                            '          , 4.000          , 5.000 (6.000 )' in
                            lines[15])
            f = open(rmsdtxt, 'r')
            lines = f.readlines()
            f.close()
            self.assertTrue(lines[0].startswith('Target_PDBID'))
            self.assertTrue(lines[2].startswith('Summary Statistics'))
            self.assertTrue(lines[4].startswith('Number_of_cases'))
            self.assertTrue('2                3                1'
                            '                1                1' in lines[4])
            self.assertTrue(lines[5].startswith('Average'))
            self.assertTrue(lines[6].startswith('Maximum'))
            self.assertTrue(lines[7].startswith('Minimum'))
            self.assertTrue(lines[8].startswith('Median'))
            self.assertTrue(lines[10].startswith('Individual Results'))

        finally:
            shutil.rmtree(temp_dir)

    def test_wait_and_check(self):

        # None check
        self.assertEqual(evaluate.wait_and_check(None), False)

        temp_dir = tempfile.mkdtemp()
        try:
            # Negative and 0 # of retries
            tfile = os.path.join(temp_dir, 'foo')
            open(tfile, 'a').close()
            self.assertEqual(evaluate.wait_and_check(tfile,
                                                     how_many_times=-1), False)
            self.assertEqual(evaluate.wait_and_check(tfile,
                                                     how_many_times=0), False)

            # File exists already
            self.assertEqual(evaluate.wait_and_check(tfile), True)

            nonexist = os.path.join(temp_dir, 'doesnotexist')
            # File does not exist
            self.assertEqual(evaluate.wait_and_check(nonexist, timestep=0),
                             False)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_distance(self):
        res = evaluate.get_distance('1, 1, 1', '1, 6, 1')
        self.assertEqual(res, 5)
        res = evaluate.get_distance('6, 1, 1', '1, 1, 1')
        self.assertEqual(res, 5)
        res = evaluate.get_distance('1, 1, 6', '1, 1, 1')
        self.assertEqual(res, 5)

        res = evaluate.get_distance('1, 2, 5', '5, 6, 7')
        self.assertEqual(res, 6)

        res = evaluate.get_distance('1, 2, 5', '5, 6, 6.5')
        self.assertTrue(math.fabs(5.8523 - res) < 0.001)

    def test_extract_crystal_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            onefcz = '1fcz'
            self.assertEqual(evaluate.extract_crystal_file(onefcz,
                                                           temp_dir), False)

            fcdir = os.path.join(temp_dir, 'fc')
            os.makedirs(fcdir, mode=0o755)
            self.assertEqual(evaluate.extract_crystal_file(onefcz,
                                                           temp_dir), False)

            onefcfile = os.path.join(fcdir, 'pdb1fcz.ent')
            open(onefcfile, 'a').close()
            self.assertEqual(evaluate.extract_crystal_file(onefcz,
                                                           temp_dir),
                             onefcfile)
        finally:
            shutil.rmtree(temp_dir)

    def test_main(self):
        temp_dir = tempfile.mkdtemp()
        current_dir = os.getcwd()
        try:
            # test with no args which should fail
            try:
                evaluate.main(['evaluate.py'])
            except SystemExit as se:
                self.assertEqual(str(se), '2')

            # test on empty directory for all inputs
            res = evaluate.main(['evaluate.py',
                                 '--dockdir', temp_dir,
                                 '--blastnfilterdir', temp_dir,
                                 '--challengedir', temp_dir,
                                 '--outdir', temp_dir,
                                 '--pdbdb', temp_dir])

            self.assertEqual(res, 0)

        finally:
            shutil.rmtree(temp_dir)

            # so main_score in evaluate changes current working directory
            # and this could mess up other tests that depend on being
            # in the d3r source tree so this chdir has been added
            os.chdir(current_dir)

    def tearDown(self):
        pass


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python

import os
from Bio import PDB
import commands
import time
import sys
import shutil
import logging
import glob
import re
import pickle
import json
from argparse import ArgumentParser

logger = logging.getLogger('d3r.evaluate')

try:
    from openeye.oechem import *
except ImportError as e:
    sys.stderr.write('Unable to import openeye, evaluate.py will '
                     'not work ' + str(e) + '\n')

# Name of log file that is written to by logging framework
FINAL_LOG = 'final.log'


def get_distance (pos1, pos2):
    """
    Given two X, Y, Z centers in format from output of get_center() function
    using standard three dimensional distance formula:

    square root (X^2 + Y^2 + Z^2)

    :param pos1: string of format 'X, Y, Z' Where X, Y, Z are numbers
    :param pos2: string of format 'X, Y, Z' Where X, Y, Z are numbers
    :return: distance as a float
    """
    _dist_0 = float(pos1.split(",")[0])-float(pos2.split(",")[0])
    _dist_1 = float(pos1.split(",")[1])-float(pos2.split(",")[1])
    _dist_2 = float(pos1.split(",")[2])-float(pos2.split(",")[2])
    return (_dist_0**2 + _dist_1**2 + _dist_2**2)**0.5


def get_center(ligand_pdb):
    """
    Parses a file that has atoms in format of:

    ATOM      1  N   SER A 183      35.409   3.264 119.011  1.00 44.03           N
    ATOM      2  CA  SER A 183      34.119   3.185 118.315  1.00 32.52           C
    HETATM 1900  C1  156 A 450      34.748  18.156  88.412  1.00 17.97           C
    HETATM 1901  C2  156 A 450      35.791  17.534  87.632  1.00 16.35           C

    and for lines starting with HETATM with unique column 2 (C1 row above)
    add column 6 (34.748 column above) to x, column 7 (18.156 column above) to y,
    and column 8 (88.412 column above)
    to z. These values would then be divided by number of HETATM lines found
    with unique column 2.

    :param ligand_pdb: path to file with data like above which corresponds to
                       atoms pertaining to receptor aka target and ligand
    :return: string of format 'X, Y, Z'
             where  X is sum of column 6 divided by number of atoms aka rows
             where  Y is sum of column 7 divided by number of atoms aka rows
             where  Z is sum of column 8 divided by number of atoms aka rows
             All numbers are set via format of %8.3f
    """
    xyz_lines = open(ligand_pdb, "r").readlines()
    multi_ligand = False
    atom_list = []
    x = y = z = 0
    for xyz_line in xyz_lines:
        if "HETATM" in xyz_line:
            #logger.debug("Check the get center of this protein: %s for this ligand: %s"%(protein_file, ligname))                                                                                                               
            atom_name = xyz_line[12:16]
            if atom_name in atom_list:
                # this variable multi_ligand is never used, not sure of its purpose
                multi_ligand = True
            else:
                atom_list.append(atom_name)
                try:
                    # Example line:
                    # HETATM 1900  C1  156 A 450      34.748  18.156  88.412  1.00 17.97

                    # 34.748 would be added to x if using example line above
                    x += float(xyz_line[30:38])

                    # 18.156 would be added to y if using example line above
                    y+= float(xyz_line[38:46])

                    # 88.412 would be added to z if using example line above
                    z+= float(xyz_line[46:54])                                  
                except:                                                         
                    logger.debug("Fatal error: Cannot find the XYZ coordinate for this ligand:%s"%ligand_pdb)
                    return False                                                
    lig_center = "%8.3f, %8.3f, %8.3f"%(x/len(atom_list), y/len(atom_list), z/len(atom_list))
    logger.debug("Ligand center for this case:%s is %s"%(ligand_pdb, lig_center))        
    return lig_center                                                       


def extract_ligand_name(blastnfilter_result_txt):
    #extract the ligand name from blastnfilter result
    result_txt = open(blastnfilter_result_txt, "r")
    result_lines = result_txt.readlines()
    for result_line in result_lines:
        if "ligand," in result_line:
            ligand_name = result_line.split(",")[1].split("\n")[0].strip()
    return ligand_name


def extract_LMCSS_ligand_name (blastnfilter_result_txt):
    result_txt = open(blastnfilter_result_txt, "r")
    result_lines = result_txt.readlines()
    for result_line in result_lines:
        if "LMCSS," in result_line:
            ligand_name = result_line.split(",")[2].strip()
    return ligand_name


def mcs(ref_mol, fit_mol):
    #do the mcs search and return OEMatch object.
    #ignore Hydrogen 
    OESuppressHydrogens(fit_mol)
    OESuppressHydrogens(ref_mol)
    #set atom and bond expression                                       
    atomexpr = OEExprOpts_AtomicNumber
    bondexpr = 0
    #do the mcs search, using defined atom, bond expression options 
    mcss = OEMCSSearch( ref_mol, atomexpr, bondexpr, True)
    mcss.SetMCSFunc(OEMCSMaxAtomsCompleteCycles(1.5) )
    #create a new match object to store mcs search info.                
    new_match_list = []
    new_match_dic  = {}
    i = 0
    j = 0
    for match1 in mcss.Match(ref_mol):                                      
        i += 1                                                              
        #write out match1 molecule                                      
        mol1 = OEGraphMol()
        OESubsetMol(mol1,match1, True)                                      
        ofs1 = oemolostream("match1_%s.pdb"%i)                              
        OEWriteMolecule(ofs1, mol1)                                         
        ofs1.close()                                                        
        for match2 in mcss.Match(fit_mol):                                  
            j += 1                                                          
            check_list = []                                                 
            #write out match2 molecule                                      
            new_match = OEMatch()                                           
            mol2 = OEGraphMol()                                             
            OESubsetMol(mol2,match2, True)                                  
            ofs2 = oemolostream("match2_%s.pdb"%j)
            OEWriteMolecule(ofs2, mol2)                                     
            ofs2.close()                                                    
            for mp1, mp2 in zip(match1.GetAtoms(), match2.GetAtoms()):      
                ref_name = mp1.target.GetName().strip()                     
                fit_name = mp2.target.GetName().strip()                     
                new_match.AddPair (mp1.target, mp2.target)                  
            #store the match info
            new_match_list.append(new_match)                                
            new_match_dic[new_match] = (["match1_%s_vs_match2_%s"%(i,j), check_list ])
    return new_match_dic


def rmsd_mcss(ref_struc, fit_struc):
    #This function use the openeye mcss calculation to get the atom mapping first and then calculate RMSD, if multiple atom mapping is avaiable, the lowest RMSM will be returned 
    reffs = oemolistream()
    reffs.open(ref_struc)
    fitfs = oemolistream()
    fitfs.open(fit_struc)
    refmol = OEGraphMol()
    OEReadMolecule(reffs, refmol)
    for fitmol in fitfs.GetOEGraphMols():
        #get all possible matching 
        ss = mcs(refmol, fitmol,)
        mcss_rmsd_list = []
        match_info = []
        for mcss in ss.keys():
            #calculate the RMSD based on atom mapping
            mcss_rmsd = OERMSD(refmol, fitmol, mcss)
            mcss_rmsd_list.append(mcss_rmsd)
            match_info.append(ss[mcss])
    try:
        minimum_mcss = min(mcss_rmsd_list)
        return minimum_mcss
    except:
        return False


def wait_and_check (filename, timestep = 5, how_many_times = 100):
    """Using a while loop and sleep this function waits `how_many_times`
       for the file set by `filename` to appear on the file system
       sleeping for `timestep` seconds between retries.
       :param filename: This is the file the loop checks for existence
                        If set to None method will always return False
       :param timestep: Sleep in seconds between retries
       :param how_many_times: # of times to check, must be a value > 0
                              otherwise code will return False
    """
    if filename is None:
        logger.error('filename passed into wait_and_check is None')
        return False

    count = 0
    while count < how_many_times:
        if not os.path.isfile(filename):
            logger.debug('Sleeping ' + str(timestep) + ' waiting for file ' + filename + ' to appear')
            time.sleep(timestep)
            count = count + 1
        else:
            return True
    return False


def align_protein (template_complex, input_complex, output_complex, timestep = 5, how_many_times = 100):
    """use schrodinger binding site alignment to get the aligned structure

       specifically $SCHRODINGER/utilities/align_binding_sites
       which looks to perform pairwise superposition of multiple structures.
       First argument is reference structure.

       When align_binding_sites.py is run the following files are created, assuming
       these are first and 2nd arguments passed in:
       crystal-ligand-residueId-chainid_receptor.pdb
       LMCSS-<target id>-<candidate id>-<ligand id>.pd

       Created files:
       crystal_<ligand id>-<residue id>-<chain id>_receptor.csv
       crystal_<ligand id>-<residue id>-<chain id>_receptor-matrix.csv
       crystal_<ligand id>-<residue id>-<chain id>_receptor.log

       NOTE: if -LOCAL parameter is passed then these 2 files are also output
             crystal_<ligand id>-<residue id>-<chain id>_receptor-merged-input.maegz
             crystal_<ligand id>-<residue id>-<chain id>_receptor-align-initial.mae

       And the output_complex file is created.

       NOTE: It might be good to refactor and add -WAIT flag first to tell command to wait
             for job completion


       :param template_complex: path to file usually named crystal-ligand-residueId-chainid_receptor.pdb
       :param input_complex: path to file usually named LMCSS-<target id>-<candidate id>-<ligand id>.pdb
       :param output_complex: path to output file
    """
    try:
        running_dir = os.getcwd()
        target_dir = os.path.dirname(os.path.abspath(output_complex))
        output_complex_filename = os.path.basename(output_complex)
        #change to the target dir since the align binding site only allow to run locally
        os.chdir(target_dir)
        logger.debug("Running: PYTHONPATH= $SCHRODINGER/utilities/align_binding_sites " + template_complex +
                      " " + input_complex + " -o " + output_complex_filename)
        val = commands.getoutput("PYTHONPATH= $SCHRODINGER/utilities/align_binding_sites %s %s -o %s"%(template_complex, input_complex, output_complex_filename))
        logger.debug("output from align_binding_sites: " + str(val))
        os.chdir(running_dir)
        return wait_and_check(output_complex_filename, timestep = timestep, how_many_times = how_many_times)
    except Exception as ex:
        logger.exception("Could not align the protein %s onto template %s"%(input_complex, template_complex))


def whole_protein_align (template_complex, input_complex, output_complex, timestep = 5, how_many_times = 100):
    """use schrodinger whole structure alignment. This function calls structalign
       with template_complex as first argument and input_complex as second argument.
       The output always appears to be named:

       rot-<template_complex file name>
       rot-<input_complex file name>

       The above always seems to be written in the current working directory.
       This function takes the rot-<input_complex file name> and moves it to `output_complex`

    :param template_complex: path to file usually named crystal-ligand-residueId-chainid_receptor.pdb
    :param input_complex: path to file usually named LMCSS-<target id>-<candidate id>-<ligand id>.pdb
    :param output_complex: path to output file
    :param timestep:
    :param how_many_times:
    :return:
    """
    try:
        running_dir = os.getcwd()
        target_dir = os.path.dirname(os.path.abspath(output_complex))
        output_complex_filename = os.path.basename(output_complex)
        os.chdir(target_dir)
        val = commands.getoutput("PYTHONPATH= $SCHRODINGER/utilities/structalign %s %s"%(template_complex, input_complex))
        logger.debug('output from structalign: ' + str(val))
        rotated_protein = "rot-" + input_complex
        if wait_and_check(rotated_protein, timestep = timestep, how_many_times = how_many_times):
            commands.getoutput("mv %s %s"%(rotated_protein, output_complex))
            return True
        else:
            return False
    except Exception as ex:
        logger.exception("Could not align the protein %s onto template %s using the whole protein alignment"%(input_complex, template_complex))


def extract_ligand_from_complex (complex_pdb_file, ligand_pdb_file, ligand_info = "UNK-900"):
    """
      Reads pdb file `complex_pdb_file` and writes lines where [17:20] matches <ligand id>
      from `ligand_info` parameter and [22:26] matches <ligand residue id> from `ligand_info`
      parameter
    :param complex_pdb_file: Input pdb file to parse
    :param ligand_pdb_file:  Output pdb file to write lines matching <ligand id>
    :param ligand_info: String of format <ligand id>-<ligand residue id>
    :return:
    """
    complex_file = open(complex_pdb_file, "r")
    complex_lines = complex_file.readlines()
    complex_file.close()
    ligand_lines = []
    ligid = ligand_info.split("-")
    for complex_line in complex_lines:
        if (complex_line[17:20].strip()==ligid[0] and complex_line[22:26].strip()==ligid[1]):
            ligand_lines.append(complex_line)
    ligand_file = open(ligand_pdb_file, "w")
    ligand_file.writelines(ligand_lines)
    ligand_file.close()


def get_ligand_info_from_ligand_file (ligand_pdb_file):
    """Given a LMCSS-<target>_<target>-<ligandid>-lig.pdb
       file which is a fragment of a pdb file with data
       that looks like this:
       HETATM 1899  C1  156 A 450      34.748  18.156  88.412  1.00 17.97           C
       HETATM 1900  C2  156 A 450      35.791  17.534  87.632  1.00 16.35           C
       HETATM 1901  C3  156 A 450      36.702  18.055  86.759  1.00 15.19           C

       The code then gets ligand id from [17:20] of line
       and ligand residue id from [22:26] which is 156 and
       450 respectively from the data above.
       The for loop logs messages if non matching
       ligand ids and residues are found, but otherwise
       continues.

       :returns: string <ligand id>-<ligand residue id>
    """
    ligand_f = open(ligand_pdb_file, "r")
    ligand_lines = ligand_f.readlines()
    ligand_f.close()
    ligand_info = None
    for ligand_line in ligand_lines:
        ligand_title = ligand_line[17:20].strip()
        ligand_resnum = ligand_line[22:26].strip()
        new_ligand_info = ligand_title + "-" + ligand_resnum
        if not ligand_info:
            #get the first ligand info
            ligand_info = new_ligand_info
        else:
            if ligand_info == new_ligand_info:
                pass
            else:
                # not sure why this is here, the log
                # message is not helpful and the first
                # encountered ligand info is what is kept
                logger.info("Get multiple ligand info, need to check while will keep the first ligand info...")
                pass
    return ligand_info


def merge_two_pdb (receptor, ligand, complex_pdb):
    """
    Merges two pdb files. Currently called from create_complex() method
    from docked() class.
    Function opens `receptor` file and grabs all lines of text that do NOT
    start with:

    CONECT
    ENDMDL
    END

    and writes them to output file `complex_pdb` in same order and appends this line
    with 3 spaces and a new line:

    TER

    Function then opens `ligand` file and grabs all lines of text that do NOT
    start with:

    REMARK
    MODEL
    CONECT
    ENDMDL

    and do NOT have the word END anywhere in the line. These lines are then appended
    to `complex_pdb` file

    :param receptor: A file path usually named <docked category>-<target id>-<candidate id>_docked.pdb
    :param ligand: A file path usually named <docked category>-<target id>-<candidate id>_docked_ligand.pdb
    :param complex_pdb: usually  <docked category>-<target id>-<candidate id>_docked_ligand_complex.pdb
    :return: True upon success or False if there is an Exception raised
    """
    try:
        complex_lines = []
        f1 = open(receptor, "r")
        protein_lines = f1.readlines()
        f1.close()
        for p_line in protein_lines:
            if p_line [:6] not in ["CONECT", "ENDMDL" ] and p_line [:3] not in ["END"]:
                complex_lines.append(p_line)
        complex_lines.append("TER   \n")
        f2 = open(ligand, "r")
        ligand_lines = f2.readlines()
        f2.close()
        for l_line in ligand_lines:                               
            if l_line [:6] not in ["REMARK", "MODEL ", "CONECT", "ENDMDL"] and l_line not in ["END"]:
                complex_lines.append(l_line)
        f3 = open(complex_pdb, "w")
        f3.writelines(complex_lines)
        f3.close()
        return True
    except Exception as ex:
        logger.exception("The receptor %s and ligand %s cannot be merged into complex %s"%(receptor, ligand, complex_pdb))
        return False


def convert_ligand_format (input_ligand, output_ligand):
    """
    Runs schrodinger structconvert to convert `input_ligand` to `output_ligand`
    Invocation by docked() class uses this to convert .mol files to .pdb.
    Unfortunately this code uses commands.getoutput and does not actually check
    the exit code for failure.

    :param input_ligand: Input .mol file
    :param output_ligand: Output .pdb file
    :return: True upon success or False if there is an exception of any kind
    """
    try:
        commands.getoutput("PYTHONPATH= $SCHRODINGER/utilities/structconvert %s %s"%(input_ligand, output_ligand))
        return True
    except Exception as ex:
        logger.exception("This ligand %s cannot be convertted to %s, need to check the format"%(input_ligand, output_ligand))
        return False


def generate_ligand_and_receptor(complex_filename,ligand_filename, receptor_filename, ligand_info):
    """This function takes a pdb file `complex_filename` which would be the target ie 1fcz.pdb/ent
       and writes out 2 new pdb files, one containing just the ligand matching data in `ligand_info`

    :param complex_filename: input target pdb file
    :param ligand_filename: output file to write lines from `complex_filename` file starting with
                            ATOM or HETATM and with matching `ligand_info` data
    :param receptor_filename: output pdb file to write lines from `complex_filename` file starting with
                            ATOM in addition any lines written to ligand_filename will be written here
    :param ligand_info: string in format of <ligand name>-<ligand residue number>-<chain id>
                        in case of 1fcz the ligand was 156-450-A
    """
    complex_f = open(complex_filename, "r")
    complex_lines = complex_f.readlines()
    complex_f.close()
    ligand_lines = []
    receptor_lines = []
    for line in complex_lines:
        # Example line:
        # HETATM 1900  C1  156 A 450      34.748  18.156  88.412  1.00 17.97           C
        #
        # In above 17:20 is 156, 20:22 is A and 22:26 is 450
        #
        if 'ATOM' in line[:4] or 'HETATM' in line[:6]:
            ligand_name = line[17:20].strip()
            ligand_chain = line[20:22].strip()
            ligand_resnum = line[22:26].strip()
            logger.debug("The ligand info for the atom in the complex is %s-%s-%s"%(ligand_name, ligand_resnum, ligand_chain))
            if "%s-%s-%s"%(ligand_name, ligand_resnum, ligand_chain)  == ligand_info:
                logger.debug("find a ligand which fits the ligand info")
                ligand_lines.append(line)
                #receptor will have the target ligand 
                receptor_lines.append(line)
            else:
                #if not "%s-%s-%s"%(ligand_name, ligand_resnum, ligand_chain) in rest_ligand_info:
                if not 'HETATM' in line[:6]:
                    #receptor will append all atoms except the other ligand
                    receptor_lines.append(line)
                else:
                    pass
    receptor_f = open(receptor_filename, "w")
    receptor_f.writelines(receptor_lines)
    receptor_f.close()
    ligand_f = open(ligand_filename, "w")
    ligand_f.writelines(ligand_lines)
    ligand_f.close()


class crystal (object):
    """Represents a crystal
    """

    def __init__(self, crystal_file, ligand_name, pdbid):
        """

        :param crystal_file: file name only invocation seens uses crystal.pdb
        :param ligand_name: name of ligand ie 156. For standard, it was obtained
                            from
                            stage.3.blastnfilter/1fcz.txt from ligand, line.
        :param pdbid: target_name for gold standard its 1fcz
        """
        self._crystal = crystal_file
        self._ligand_name = ligand_name
        # in only invocation seen just crystal.pdb is passed in so
        # code assumes file is in working directory and path will
        # be set to ''
        #
        self._crystal_path = os.path.dirname(self._crystal)

        # since crystal.pdb is passed in this will be unchanged
        # and set to crystal.pdb
        #
        self._crystal_name = os.path.basename(self._crystal)
        parser = PDB.PDBParser()

        # writer is never used
        writer = PDB.PDBIO()
        self._biostruc = parser.get_structure (pdbid, crystal_file)

    def get_ligand_info (self):
        """
          This method builds an internal list of residues from
          the crystal file passed in via the constructor using
          Biopython's PDB.PDBParser(). The list only contains
          entries whose residue name matches ligand_name passed
          in via the constructor.

          The contents of the list is are strings of this format:
          <residue name>-<residue id>-<chain id>

          Duplicate entries are ignored except for a message to debug
          of logger
        """
        #extract the target ligand information
        self._ligand_residue_info = []
        for _model in self._biostruc:
            for _chain in _model:
                for _residue in _chain:   
                    _residue_ligand_name = _residue.get_resname()
                    if _residue_ligand_name == self._ligand_name: 
                        _residue_info = "%s-%s-%s"%(_residue.get_resname(), _residue.get_id()[1], _chain.get_id())
                        if _residue_info not in self._ligand_residue_info:
                            self._ligand_residue_info.append(_residue_info)
                            logger.debug("Get the residue info :%s"%_residue_info)
                        else:
                            logger.debug("Got multiple ligand with the same residue info %s"%_residue_info)
                            pass
                    else:
                        pass

    def get_ligand_and_receptor_files(self):
        #get the ligand pdb file and receptor file per ligand 
        self._ligand = []

        # self._receptor is directly read by LMCSS.align_LMCSS_onto_crystal
        # and by docked.align_complex_onto_crystal
        #
        self._receptor = []
        self._valid_ligand_info = []
        if len(self._ligand_residue_info) > 0:
            for _ligand_residue_info in self._ligand_residue_info:
                #get the ligand info for the rest of the ligand other than the target ligand and remove all of those ligand when generate the receptor
                _crystal_filename, _crystal_file_extension = os.path.splitext(self._crystal_name)
                _ligand_filename = _crystal_filename + "_" + _ligand_residue_info + "_" + "ligand" + _crystal_file_extension
                _receptor_filename = _crystal_filename + "_" + _ligand_residue_info + "_" + "receptor" + _crystal_file_extension  
                #store the ligand and receptor at the same path folder as the crystal path
                _ligand_file_full_path = os.path.join(self._crystal_path, _ligand_filename)
                _receptord_file_full_path = os.path.join(self._crystal_path, _receptor_filename)
                try:
                    # generate_ligand_and_receptor
                    # writes out crystal_ligand-residueId-chainid_ligand.pdb
                    #            crystal_ligand-residueId-chainid_receptor.pdb
                    # which contain just the atoms for the ligand (in _ligand file) and atoms
                    # for both target and ligand in the _receptor file)
                    generate_ligand_and_receptor (self._crystal, _ligand_file_full_path, _receptord_file_full_path, _ligand_residue_info )
                    logger.debug("Generating the ligand %s and receptor %s "%(_ligand_file_full_path, _receptord_file_full_path))
                    self._ligand.append(_ligand_file_full_path)
                    self._receptor.append(_receptord_file_full_path)
                    self._valid_ligand_info.append(_ligand_residue_info)
                except Exception as ex:
                    logger.exception("Failed to generate the ligand and receptor")
                    pass
        else:
            logger.info("There is no valid ligand in the crystal file, pass the ligand and receptor extraction step") 
            pass


class LMCSS (object):
    """initial LMCSS complex released to the participant, used to check
     the distance of LMCSS ligand with the crystal ligand
    """
     
    def __init__(self, LMCSS_complex, ligand_info, crystal_obj):
        """
        Constructor.
        :param LMCSS_complex: Path to LMCSS-<target id>-<candidate id>-<ligand id>.pdb which
                              contains pdb for both ligand and I think candidate
        :param ligand_info: Should be a string in format <ligand id>-<ligand residue id>
        :param crystal_obj: Instance of crystal class loaded with <target id>
        """
        self._LMCSS = LMCSS_complex 
        self._LMCSS_path = os.path.dirname(self._LMCSS)
        self._LMCSS_name = os.path.basename(self._LMCSS)

        # this variable is LMCSS_complex file name only with .pdb suffix stripped off
        self._LMCSS_basename = os.path.splitext(self._LMCSS_name)[0]
         
        self._ligand = ligand_info
        self._crystal = crystal_obj

    def align_LMCSS_onto_crystal(self, time_check_frequence = 5, check_point_number = 100,):
        self._all_aligned_complex = {}
        self._all_aligned_ligand = {}
        if self._LMCSS:

            # self._crystal._receptor contains a list of file paths to files
            # with name crystal-ligand-residueId-chainid_receptor.pdb
            # which contain atoms for target and ligand
            #
            for _crystal_receptor_index, _crystal_receptor in enumerate(self._crystal._receptor):

                # self._crystal._valid_ligand_info is a list filled with
                # strings of format <residue name>-<residue id>-<chain id>
                # and is filled by crystal.get_ligand_info() and
                # by crystal.get_ligand_and_receptor_files()
                # the enumeration requires the order to be consistent
                # with self._crystal._receptor
                #
                _crystal_ligand_info = self._crystal._valid_ligand_info[_crystal_receptor_index]
                logger.debug("Try to align the docked ligand onto this crystal receptor:%s"%(_crystal_receptor))


                # _aligned_complex_name is set to
                # LMCSS-<target id>-<candidate id>-<ligand id>_aligned_<residue name>-<residue id>-<chain id>.pdb
                #
                self._aligned_complex_name = self._LMCSS_basename + "_aligned" + "_" + _crystal_ligand_info + ".pdb"

                # _aligned_ligand_name is set to
                # LMCSS-<target id>-<candidate id>-<ligand id>_aligned_ligand_<residue name>-<residue id>-<chain id>.pdb
                #
                self._aligned_ligand_name = self._LMCSS_basename + "_aligned_ligand" + "_" + _crystal_ligand_info + ".pdb"

                #
                # prepend directory which from code looks to be outdir/<target>/score/
                #
                self._aligned_complex = os.path.join(self._LMCSS_path, self._aligned_complex_name)
                self._aligned_ligand = os.path.join(self._LMCSS_path, self._aligned_ligand_name)

                #do the alignment
                logger.debug("Try to align the original LMCSS %s onto the crystal receptor %s"%(self._LMCSS, _crystal_receptor))
                #try the binding site alignment first
                # below align_binding_sites is tried and then structalign
                #
                if not align_protein(_crystal_receptor, self._LMCSS, self._aligned_complex, timestep = time_check_frequence, how_many_times = check_point_number ):
                    #go to whole protein alignment
                    total_relaxing_time = time_check_frequence*check_point_number
                    logger.debug("The binding site alignment from %s onto %s didn't finish in %s second... Need to break"%(self._LMCSS, _crystal_receptor, total_relaxing_time))
                    if not whole_protein_align(_crystal_receptor, self._LMCSS, self._aligned_complex, timestep = time_check_frequence, how_many_times = check_point_number ):
                        total_relaxing_time = time_check_frequence*check_point_number
                        logger.info("The whole protein alignment from %s onto %s didn't finish in %s second... Need to break"%(self._LMCSS, _crystal_receptor, total_relaxing_time))
                    else:
                        # _crystal_ligand_info is a string of format
                        # <residue name>-<residue id>-<chain id>
                        # and its being used as a key to map to the aligned output
                        # LMCSS-<target id>-<candidate id>-<ligand id>_aligned_<residue name>-<residue id>-<chain id>.pdb
                        #
                        self._all_aligned_complex[_crystal_ligand_info] = self._aligned_complex
                else:
                    # _crystal_ligand_info is a string of format
                    # <residue name>-<residue id>-<chain id>
                    # and its being used as a key to map to the aligned output
                    # LMCSS-<target id>-<candidate id>-<ligand id>_aligned_<residue name>-<residue id>-<chain id>.pdb
                    #
                    self._all_aligned_complex[_crystal_ligand_info] = self._aligned_complex
                logger.debug("Successfully align %s onto %s and get the aligned structure %s"%(self._LMCSS, _crystal_receptor, self._aligned_complex))
                #extract the ligand from aligned complex
                try:
                    # extract ligand from output pdb and save to self._aligned_ligand file name
                    # LMCSS-<target id>-<candidate id>-<ligand id>_aligned_ligand_<residue name>-<residue id>-<chain id>.pdb
                    extract_ligand_from_complex(self._aligned_complex, self._aligned_ligand, ligand_info = self._ligand)
                    logger.debug("Successfully extract ligand file %s from aligned complex %s"%(self._aligned_ligand, self._aligned_complex))
                    self._all_aligned_ligand[_crystal_ligand_info] = self._aligned_ligand
                except Exception as ex:
                    logger.exception("Cannot extract ligand file %s from aligned complex %s"%(self._aligned_ligand, self._aligned_complex))

    def calculate_distance(self):
        #calculate the distance between LMCSS ligand and all possible crystal ligand
        # contains a dictionary of
        # <residue name>-<residue id>-<chain id> => <distance between centers>
        self._dis = {}

        # contains smallest distance found in self._dis
        # and is referenced in main_score() around line 1133
        #
        self._min_dis = None 
        if self._all_aligned_ligand:
            # self._crystal._ligand contains a list of file paths to files
            # with name crystal-ligand-residueId-chainid_ligand.pdb
            # which contain atoms for ligand
            #
            for _crystal_ligand_index, _crystal_ligand in enumerate(self._crystal._ligand):
                # self._crystal._valid_ligand_info is a list filled with
                # strings of format <residue name>-<residue id>-<chain id>
                # and is filled by crystal.get_ligand_info() and
                # by crystal.get_ligand_and_receptor_files()
                # the enumeration requires the order to be consistent
                # with self._crystal._receptor
                #
                _crystal_ligand_info = self._crystal._valid_ligand_info[_crystal_ligand_index]
                if _crystal_ligand_info in self._all_aligned_ligand:
                    _docked_ligand_aligned = self._all_aligned_ligand[_crystal_ligand_info]
                    try:
                        _crystal_ligand_center = get_center(_crystal_ligand)
                        _docked_ligand_aligned_center = get_center(_docked_ligand_aligned)
                        _distance = get_distance(_crystal_ligand_center, _docked_ligand_aligned_center)
                        # build a map of <residue name>-<residue id>-<chain id> => <distance between centers>
                        self._dis[_crystal_ligand_info] = _distance

                        logger.debug("The distance for %s vs %s is %s"%(_crystal_ligand, _docked_ligand_aligned, _distance))
                    except Exception as ex:
                        logger.exception("The distance calculation for %s vs %s failed"%(_crystal_ligand, _docked_ligand_aligned))

                # I'm guessing this can be taken out of this for loop and calculated just once
                if self._dis:
                    self._min_dis = sorted(self._dis.values())[0]


class docked (object):
    #need to name the title as LMCSS, SMCSS etc
    #1, merge ligand and receptor
    #2, align the complex onto the crystal receptor
    #3, calculate the RMSD for each aligned ligand 
    def __init__(self, docked_ligand_filename, docked_receptor_filename, docked_category, crystal_obj):
        """
        Constructor
        :param docked_ligand_filename: filename of user submission of mol file
                                       usually in format like so;
                                       <docked category>-<target id>-<candidate id>_docked.mol
        :param docked_receptor_filename: filename of user submission in pdb file
                                       usually in format like so;
                                       <docked category>-<target id>-<candidate id>_docked.pdb
        :param docked_category: string usually set to one of following;
                                LMCSS, SMCSS, hiResApo, hiResHolo
        :param crystal_obj: instance of crystal class properly loaded with target
        """

        # sets _docked_type to LMCSS, SMCSS, hiResApo, or hiResHolo
        self._docked_type = docked_category

        # set to <docked category>-<target id>-<candidate id>_docked.mol
        self._docked_ligand_mol = docked_ligand_filename

        # directory where mol file resides, but this will be empty since
        # code calling this strips off the path before calling this
        # constructor
        self._docked_ligand_mol_path = os.path.dirname(self._docked_ligand_mol)

        # just name of file, but will be same cause code calling this already
        # strips off path
        self._docked_ligand_mol_name = os.path.basename(self._docked_ligand_mol)

        # strip off extension and put base name in self._docked_ligand_mol_basename and put
        # extension in self._docked_ligand_mol_extension
        self._docked_ligand_mol_basename, self._docked_ligand_mol_extension = os.path.splitext(self._docked_ligand_mol_name)

        # set to <docked category>-<target id>-<candidate id>_docked.pdb
        self._docked_receptor_pdb = docked_receptor_filename

        # set to empty string cause main_score strips off path already
        self._docked_receptor_pdb_path = os.path.dirname(self._docked_receptor_pdb)

        # set to same as self._docked_receptor_pdb
        self._docked_receptor_pdb_name = os.path.basename(self._docked_receptor_pdb)

        # strip of extension and put base name into self._docked_receptor_pdb_basename
        # and put extension in self._docked_receptor_pdb_extension
        self._docked_receptor_pdb_basename, self._docked_receptor_pdb_extension = os.path.splitext(self._docked_receptor_pdb_name)
        #check 1, if the path of ligand and the path of receptor are the same
        
        self._docked_ligand_pdb = None

        # set to crystal class instance
        self._crystal = crystal_obj

        # code merely logs if extensions are incorrect
        # with no real checks
        if self._docked_ligand_mol_extension != ".mol":
            logger.debug ("This docked ligand %s is not in mol file format"%self._docked_ligand_mol)
        elif self._docked_receptor_pdb_extension != ".pdb":
            logger.debug ("This docked receptor %s is not in pdb file format"%self._docked_receptor_pdb)
        else:
            #
            # convert to pdb file using schroding convert_ligand_format
            # by creating new file name by appending _ligand.pdb to basename of mol
            # file which is <docked category>-<target id>-<candidate id>_docked
            #
            self._docked_ligand_pdb_name = self._docked_ligand_mol_basename + "_ligand.pdb"

            # make sure the pdb format file has the same path with the mol format file
            self._docked_ligand_pdb = os.path.join(self._docked_ligand_mol_path, self._docked_ligand_pdb_name)

            # convert mol to pdb and only set self._docked_ligand_pdb to None if there is a
            # failure.
            #
            if not convert_ligand_format(self._docked_ligand_mol, self._docked_ligand_pdb):
                self._docked_ligand_pdb = None
            
    def create_complex (self):
        """
        Merges creates <docked category>-<target id>-<candidate id>_docked_ligand_complex.pdb
        by merging <docked category>-<target id>-<candidate id>_docked_ligand.pdb with
        docked category>-<target id>-<candidate id>_docked.pdb
        """
        self._docked_complex = None
        if self._docked_ligand_pdb:
            #merge the receptor and the ligand together
            self._docked_complex_name = self._docked_ligand_pdb_name + "_complex.pdb"
            self._docked_complex = os.path.join(self._docked_ligand_mol_path, self._docked_complex_name)
            if not merge_two_pdb(self._docked_receptor_pdb, self._docked_ligand_pdb, self._docked_complex):
                self._docked_complex = None

    def align_complex_onto_crystal(self, time_check_frequence=5,
                                   check_point_number = 100,
                                   sc_ligand_default = "UNK-900"):
        self._all_aligned_complex = {} 
        self._all_aligned_ligand = {} 
        if self._docked_complex:
            # self._crystal._receptor contains a list of file paths to files
            # with name crystal_ligand-residueId-chainid_receptor.pdb
            # which contain atoms for target and ligand
            #
            for _crystal_receptor_index, _crystal_receptor in enumerate(self._crystal._receptor):
                # self._crystal._valid_ligand_info is a list filled with
                # strings of format <residue name>-<residue id>-<chain id>
                # and is filled by crystal.get_ligand_info() and
                # by crystal.get_ligand_and_receptor_files()
                # the enumeration requires the order to be consistent
                # with self._crystal._receptor and self._crystal._ligand
                #
                _crystal_ligand_info = self._crystal._valid_ligand_info[_crystal_receptor_index]
                logger.debug("Try to align the docked ligand onto this crystal receptor:%s"%(_crystal_receptor))

                # set to <docked category>-<target id>-<candidate id>_docked_complex_aligned_<residue name>-<residue id>-<chain id>.pdb
                self._aligned_complex_name = self._docked_ligand_mol_basename + "_complex_aligned" + "_" + _crystal_ligand_info + ".pdb"

                # set to <docked category>-<target id>-<candidate id>_docked_complex_aligned_ligand_<residue name>-<residue id>-<chain id>.pdb
                self._aligned_ligand_name = self._docked_ligand_mol_basename + "_complex_aligned_ligand" + "_" + _crystal_ligand_info + ".pdb"

                # prepend path to self._aligned_complex_name
                self._aligned_complex = os.path.join(self._docked_ligand_mol_path, self._aligned_complex_name)

                # prepend path to self._aligned_ligand_name
                self._aligned_ligand = os.path.join(self._docked_ligand_mol_path, self._aligned_ligand_name)

                #do the alignment
                logger.debug("Try to align the docked complex %s onto the crystal receptor %s"%(self._docked_complex, _crystal_receptor))

                # try the binding site alignment first with
                # _crystal_receptor set to crystal_ligand-residueId-chainid_receptor.pdb
                # self._docked_complex set to <docked category>-<target id>-<candidate id>_docked_ligand_complex.pdb
                # self._aligned_complex set to <docked category>-<target id>-<candidate id>_docked_complex_aligned_<residue name>-<residue id>-<chain id>.pdb
                # if align_protein() fails try whole_protein_align()
                #
                if not align_protein(_crystal_receptor, self._docked_complex, self._aligned_complex, timestep = time_check_frequence, how_many_times = check_point_number ):
                    #go to whole protein alignment
                    total_relaxing_time = time_check_frequence*check_point_number
                    logger.debug("The binding site alignment from %s onto %s didn't finish in %s second... Need to break"%(self._docked_complex, _crystal_receptor, total_relaxing_time))
                    if not whole_protein_align(_crystal_receptor, self._docked_complex, self._aligned_complex, timestep = time_check_frequence, how_many_times = check_point_number ):
                        total_relaxing_time = time_check_frequence*check_point_number
                        logger.info("The whole protein alignment from %s onto %s didn't finish in %s second... Need to break"%(self._docked_complex, _crystal_receptor, total_relaxing_time))
                    else:
                        self._all_aligned_complex[_crystal_ligand_info] = self._aligned_complex
                else:
                    self._all_aligned_complex[_crystal_ligand_info] = self._aligned_complex


                logger.debug("Successfully align %s onto %s and get the aligned structure %s"%(self._docked_complex, _crystal_receptor, self._aligned_complex))
                #extract the ligand from aligned complex
                try:
                    # extract ligand from output pdb and save to self._aligned_ligand file name
                    # <docked category>-<target id>-<candidate id>_docked_complex_aligned_ligand_<residue name>-<residue id>-<chain id>.pdb
                    extract_ligand_from_complex(self._aligned_complex, self._aligned_ligand, ligand_info = sc_ligand_default)
                    logger.debug("Successfully extract ligand file %s from aligned complex %s"%(self._aligned_ligand, self._aligned_complex))
                    self._all_aligned_ligand[_crystal_ligand_info] = self._aligned_ligand
                except Exception as ex:
                    logger.exception("Cannot extract ligand file %s from aligned complex %s"%(self._aligned_ligand, self._aligned_complex))

    def calculate_rmsd_and_distance (self):
        # 
        self._rmsd_dis = {}
        self._min_rmsd_dis = None
        if self._all_aligned_ligand:
            for _crystal_ligand_index, _crystal_ligand in enumerate(self._crystal._ligand):
                _crystal_ligand_info = self._crystal._valid_ligand_info[_crystal_ligand_index]
                if _crystal_ligand_info in self._all_aligned_ligand:
                    _docked_ligand_aligned = self._all_aligned_ligand[_crystal_ligand_info]
                    try:
                        #_rmsd, _distance = rmsd_mcss (_crystal_ligand, _docked_ligand_aligned)
                        _rmsd = rmsd_mcss (_crystal_ligand, _docked_ligand_aligned)
                        _crystal_ligand_center = get_center(_crystal_ligand)
                        _docked_ligand_aligned_center = get_center(_docked_ligand_aligned) 
                        _distance = get_distance(_crystal_ligand_center, _docked_ligand_aligned_center)
                        _all_mapping_files = glob.glob("match*.pdb")
                        for _mapping_file in _all_mapping_files:
                            os.remove(_mapping_file)
                        self._rmsd_dis[_crystal_ligand_info] = (_rmsd, _distance)
                        logger.debug("The rmsd and distance for %s vs %s is %s"%(_crystal_ligand, _docked_ligand_aligned, _rmsd))
                    except Exception as ex:
                        logger.exception("The rmsd calculation for %s vs %s failed"%(_crystal_ligand, _docked_ligand_aligned))
            #get the lowest RMSD and the corresponding distance
            if self._rmsd_dis:
                self._min_rmsd_dis = sorted(self._rmsd_dis.values())[0]


def clean_up_list_of_value(list_of_value):
    """Given list this method creates a
    new list removing any items that have
    value of None
    :returns: list of values in same order with None
              values removed
    """
    #clean up all None values
    new_list = []
    if list_of_value is None:
        return new_list

    for item in list_of_value:
        if item:
            new_list.append(item)
    return new_list


def calculate_average_min_max_median(full_list_of_value):
    """Given a list of values in `full_list_of_value`
    generate basic stats about values in list
    :returns: tuple of strings (average, min, max, median)
    """
    list_of_value = clean_up_list_of_value(full_list_of_value)
    if list_of_value is not None and len(list_of_value) > 0:
        count = len(list_of_value)
        average = "%-15.3f, " % (float(sum(list_of_value))/count)

        if count == 1:
            median = "%-15.3f, " % list_of_value[0]
        else:
            sorted_list = sorted(list_of_value)
            if count % 2 == 0:
                median_index = count / 2
                median = "%-15.3f, " % float(float(sum(sorted_list[median_index-1:median_index+1]))/2)
            else:
                median = "%-15.3f, " % sorted_list[int(round(count / 2))]

        minimum = "%-15.3f, " % min(list_of_value)
        maximum = "%-15.3f, " % max(list_of_value)
    else:
        average = minimum = maximum = median = "%-15s, " % " "

    return average, minimum, maximum, median


class DataContainer(object):

    def __init__(self, ):
        self._data = {}

    def get_all_docked_type(self, docked_type="LMCSS"):
        """Create list of values by examining internal
           data structure of class and for `docked_type` passed
           in return values from all targets registered via
           `register` method.
           :param docked_type: extract values corresponding to this docked_type
           :returns list of values
           """
        docked_type_value = []
        if self._data:
            for target_ID in self._data:
                if docked_type in self._data[target_ID]:
                    docked_type_value.append(self._data[target_ID]
                                             [docked_type])
        return docked_type_value

    def register(self, target_ID, docked_type, value):
        #the docked type could be either "LMCSS" or "LMCSS_dis"
        # the value could be either rmsd or distance
        if value or value == 0.0:
            if target_ID not in self._data:
                self._data[target_ID] = {}
            if docked_type:
                if docked_type not in self._data[target_ID]:
                    self._data[target_ID][docked_type] = value

    def layout_pickle( self, pickle_filename = "RMSD.pickle"):
        self._pf = open(pickle_filename, "w")
        pickle.dump(self._data, self._pf)
        self._pf.close()

    def layout_json(self, json_filename = "RMSD.json"):
        self._jf = open(json_filename,'w')
        json.dump(self._data, self._jf)
        self._jf.close()

    def layout_plain (self,  plain_filename = "RMSD"):
        self._plain_filename = plain_filename
        self._combined_csv_filename = self._plain_filename + ".csv" 
        self._combined_txt_filename = self._plain_filename + ".txt" 
        self._LMCSS_list = self.get_all_docked_type(docked_type="LMCSS")
        self._SMCSS_list = self.get_all_docked_type(docked_type="SMCSS")
        self._hiResApo_list = self.get_all_docked_type(docked_type="hiResApo")
        self._hiResHolo_list = self.get_all_docked_type(docked_type="hiResHolo")
        self._hiTanimoto_list = self.get_all_docked_type(docked_type="hiTanimoto")

        self._whole_data_lines = ["%-20s%-15s, %-15s, %-15s, %-15s, %-15s, "
                                  "%-15s \n" % ("Target_PDBID,", "LMCSS", "SMCSS",
                                                "hiResApo", "hiResHolo", "hiTanimoto",
                                                "LMCSS_ori_distance")]

        self._whole_data_lines.append("\nSummary Statistics\n\n")

        #append the number of cases line
        self._whole_data_lines.append("%-20s%-15s, %-15s, %-15s, %-15s, "
                                      "%-15s, \n" % ("Number_of_cases,",
                                                     len(self._LMCSS_list),
                                                     len(self._SMCSS_list),
                                                     len(self._hiResApo_list),
                                                     len(self._hiResHolo_list),
                                                     len(self._hiTanimoto_list)))

        #append the average min max median, first
        (self._average_line, self._max_line,
         self._min_line, self._medium_line) = ("%-20s"%"Average,",
                                               "%-20s"%"Maximum,",
                                               "%-20s"%"Minimum,",
                                               "%-20s"%"Median,")

        for value_list in (self._LMCSS_list, self._SMCSS_list,
                           self._hiResApo_list, self._hiResHolo_list,
                           self._hiTanimoto_list):
            (average_score, min_score,
             max_score,
             medium_score) = calculate_average_min_max_median(value_list)

            self._average_line += average_score 
            self._max_line += max_score 
            self._min_line += min_score 
            self._medium_line += medium_score
        self._average_line += "\n" 
        self._max_line += "\n" 
        self._min_line += "\n" 
        self._medium_line += "\n" 
        self._whole_data_lines.append(self._average_line)
        self._whole_data_lines.append(self._max_line)
        self._whole_data_lines.append(self._min_line)
        self._whole_data_lines.append(self._medium_line)
        self._whole_data_lines.append("\nIndividual Results\n\n")
        #append the value for each PDBID 
        all_pro_type = ["LMCSS", "SMCSS", "hiResApo", "hiResHolo",
                        "hiTanimoto", "LMCSS_ori"]
        for target_ID in self._data:
            self._new_line_score = ""
            self._valid_line = False
            for docked_type in all_pro_type:
                if docked_type in self._data[target_ID]:
                    docked_type_dis = docked_type + "_dis"
                    if docked_type_dis in self._data[target_ID]:
                        self._new_line_score += "%-6.3f(%-6.3f) , " % (self._data[target_ID][docked_type],
                                                                       self._data[target_ID][docked_type_dis])
                    else:
                        if not docked_type == "LMCSS_ori":
                            self._new_line_score += "%-15.3f, " % (self._data[target_ID][docked_type])
                        else:
                            self._new_line_score += "(%-6.3f)       , " % (self._data[target_ID][docked_type])
                    self._valid_line = True
                else:
                    self._new_line_score += "%-15s, " % (" ")
            if self._valid_line:
                pdbid_full = "%s," % target_ID
                self._new_line = "%-20s" % (pdbid_full)
                self._new_line += self._new_line_score
                self._new_line += "\n"
                self._whole_data_lines.append(self._new_line)
        #write out csv file
        self._plain_csv_file = open(self._combined_csv_filename, "w")
        self._plain_csv_file.writelines(self._whole_data_lines)
        self._plain_csv_file.close()
        #write out txt file
        self._whole_data_lines_txt = [item.replace(",",
                                                   " ") for item in self._whole_data_lines]
        self._plain_txt_file = open(self._combined_txt_filename, "w")
        self._plain_txt_file.writelines(self._whole_data_lines_txt)
        self._plain_txt_file.close()


def calculate_rmsd(crystal_obj, docked_lig_mol, docked_receptor, docked_structure_type):
    docked_obj = docked(docked_lig_mol, docked_receptor, docked_structure_type, crystal_obj)
    docked_obj.create_complex()
    docked_obj.align_complex_onto_crystal(check_point_number = 10)
    docked_obj.calculate_rmsd_and_distance()
    return docked_obj


def create_crystal_obj (crystal_file, ligand_name, crystal_id):
    crystal_obj = crystal(crystal_file, ligand_name, crystal_id)
    crystal_obj.get_ligand_info()
    crystal_obj.get_ligand_and_receptor_files()
    return crystal_obj


def get_submitted_file_list (file_pattern_name = "docked.mol"):
    all_mol_files = glob.glob("*%s"%file_pattern_name)
    all_receptor_files = []
    if all_mol_files:
        for mol_file in all_mol_files:
            receptor_file = mol_file.replace('.mol','.pdb')
            all_receptor_files.append(receptor_file)
    return (all_mol_files, all_receptor_files)


def extract_crystal_file (crystal_pdbid, crystal_path):
    crystal_middle_name = crystal_pdbid[1:3]
    crystal_ent_file = "pdb" + crystal_pdbid + ".ent"
    crystal_pdbloc = os.path.join(crystal_path, crystal_middle_name, crystal_ent_file)
    if os.path.isfile(crystal_pdbloc):
        return crystal_pdbloc
    else:
        return False


def main_score(dock_dir, pdb_protein_path, evaluate_dir, blastnfilter_dir, challenge_data_path, update= True):
    """
    Main entry function
    :param dock_dir:  Path to dock submission.
                      Ex: /..../2018/data...12/stage.6.123_foo.extsubmission
    :param pdb_protein_path: Path to uncompressed pdb
                             Ex: /celpp/pdb.extracted
    :param evaluate_dir: Output directory to use.
                         Ex: /..../stage.7.123_foo.extsubmission.evaluation
    :param blastnfilter_dir: Path to blastnfilter stage
                         Ex: /..../stage.3.blastnfilter
    :param challenge_data_path: Path to challenge data package uncompressed
                                directory
                        Ex: /..../stage.4.challengedata/celpp_week12_2018

    :param update:
    :return:
    """
    #1 copy docked dir info into evaluate dir and create score for targe
    #2 create score dir and copy the ideal files into score dir
    #3 create submit obj and crystal obj in the crystal obj, and store the cyrstal obj directly 
    #4 after each calculation, append the crystal obj to the final result file in the upper level path
    #5 if for individual folder, the crystal obj is empty, also save it and skip it and go to the next case
    #1 switch to evaluation folder
    os.chdir(evaluate_dir)
    current_dir = os.getcwd()

    # DataContainer() is a class to hold the results of the evaluation
    #                  Data is added via register() method and
    #                  the information is held in a dictionary and
    #                  persisted to various files via the layout_json(),
    #                  layout_pickle(), and layout_plain() methods
    result_container = DataContainer()
    result_pickle = "RMSD.pickle"
    pickle_full_path = os.path.join(current_dir, result_pickle) 
    result_json = "RMSD.json"
    json_full_path = os.path.join(current_dir, result_json) 
    result_plain = "RMSD"
    plain_full_path = os.path.join(current_dir, result_plain)

    #get all target dir info from the docked dir
    #
    # This giant for loop iterates through all directories
    # in dock_dir which is stage.6.###_XXX.extsubmission directory
    # and the target_dir is just the directory name. Ex: 1fcz
    target_dirs = list(os.walk(dock_dir))[0][1]
    for target_dir in target_dirs:
        #valid target must have at least one RMSD value return, else this target is invalid
        #
        # This variable is set to True below but never queried or used
        # in the code
        #
        valid_target = False
        
        #copy the target dir to the evaluate dir
        # commands.getoutput("cp -r %s/%s %s"%(dock_dir,target_dir, evaluate_dir))
        #
        # My attempt to replace commands.getoutput with shutil. Looks
        # like the dock_dir/target_dir is copied to the output directory recursively
        # Not sure why since the files could just be referenced.

        target_dir_path = os.path.join(dock_dir, target_dir)
        evaluate_target_dir = os.path.join(evaluate_dir, target_dir)
        logger.debug("Recursively copying " + target_dir_path + " to " + evaluate_target_dir)
        shutil.copytree(target_dir_path, evaluate_target_dir)

        #
        # target_name is totally not needed since target_dir is already
        # the directory name
        #
        target_name = os.path.basename(target_dir)
        logger.info("=================We start to work on this target %s================="%target_name)
        #2 switch into the individual case
        os.chdir(target_name)
        #
        # get_submitted_file_list() looks for all *docked.mol in the directory
        # and generates a tuple (*docked.mol, *docked.pdb)
        #
        all_mol_files, all_receptor_files = get_submitted_file_list()
        if not all_mol_files:
            logger.info("For this folder %s, there is no valid docked structure"%target_dir)
            result_container.register(target_dir, docked_type = None, value = None)
            ######need to register into the result pickle and txt, csv 
            os.chdir(current_dir)
            continue
        else:
            os.mkdir("score")
            # For some reason naming target_name aka target_dir crystal_ID
            #
            #copy the crystal structure first
            crystal_ID = target_name
            #extract the crystal ligand name from blastnfilter result txt file

            # Look at target_dir.txt from stage.3.blastnfilter directory
            # and get the value to right of comma from the line starting with ligand,
            # set that value to "ligand_name" variable below
            #
            blastnfilter_result_name = crystal_ID + ".txt"
            blastnfilter_txt = os.path.join(blastnfilter_dir, blastnfilter_result_name)
            ligand_name = extract_ligand_name(blastnfilter_txt) 
            LMCSS_ligand_name = extract_LMCSS_ligand_name(blastnfilter_txt)

            # copy target_dir aka target_name aka crystal_ID .ent file from pdb
            # directory ie /celpp/pdb.extracted/XX/pdbXXXX.ent
            # and copy to score/crystal.pdb
            #
            #get crystal structure location
            crystal_file = extract_crystal_file(crystal_ID, pdb_protein_path)
            if crystal_file:
                pdbid_local_path = os.getcwd()
                commands.getoutput("cp %s score/crystal.pdb"%(crystal_file))
                try:
                    os.chdir("score")

                    # From score directory call create_crystal_obj which creates a
                    # crystal object. See crystal class above for more information
                    # but in summary creates the following:
                    # target_name/score/crystal.pdb
                    # target_name/score/crystal_ligand-residueId-chainid_ligand.pdb
                    # target_name/score/crystal_ligand-residueId-chainid_receptor.pdb
                    #
                    # Where
                    # crystal.pdb is just the target pdb file obtained from
                    #             pdb database
                    # crystal..._ligand.pdb contains the atoms for the ligand obtained
                    #                       by extracting them out of the crystal.pdb
                    #
                    # crystal..._receport.pdb contains atoms for target and ligand extracted
                    #                         from crystal.pdb
                    #
                    crystal_obj = create_crystal_obj("crystal.pdb", ligand_name, crystal_ID)
                    logger.info("\tSuccessfully create the crystal object")

                    #calculate the distance between the LMCSS ligand and crystal ligand
                    #get the file location

                    # set LMCSS_ori_file_path to <challenge dir>/<target_name>
                    LMCSS_ori_file_path = os.path.join(challenge_data_path, crystal_ID)
                    LMCSS_complex_only_list = []
                    LMCSS_ligand_only_list = []

                    # get a list of all files with LMCSS*.pdb in <challenge dir>/<target name>
                    all_LMCSS_files = glob.glob("%s/LMCSS*.pdb"%LMCSS_ori_file_path)

                    # iterate through all files and if -lig.pdb not in the filename
                    # add it to LMCSS_complex_only_list list otherwise
                    # add it to LMCSS_ligand_only_list
                    #
                    for LMCSS_file in all_LMCSS_files:
                        if "-lig.pdb" not in os.path.basename(LMCSS_file):
                            LMCSS_complex_only_list.append(LMCSS_file)
                        else:
                            LMCSS_ligand_only_list.append(LMCSS_file)

                    # Looks like there should be 1 item in each list otherwise
                    # its an error, but all that happens in an error case is a log message is written
                    # out and code continues
                    #
                    if len(LMCSS_complex_only_list) == 1 and len(LMCSS_ligand_only_list) == 1:
                        LMCSS_complex = LMCSS_complex_only_list[0]
                        LMCSS_ligand = LMCSS_ligand_only_list[0]

                        # below code parses contents of file and returns a string of
                        # <ligand id>-<ligand residue id>
                        # putting it into ligand_info variable
                        #
                        ligand_info = get_ligand_info_from_ligand_file(LMCSS_ligand) 

                        LMCSS_local = os.path.basename(LMCSS_complex)

                        # Copy over the LMCSS*.pdb file that does NOT have -lig.pdb in its name
                        # Usually named: LMCSS-<target>_<target>-<ligand id>.pdb
                        #
                        #copy locally
                        commands.getoutput("cp %s %s"%(LMCSS_complex, LMCSS_local))
                        #create the LMCSS obj
                        # and pass in LMCSS-<target>_<target>-<ligand id>.pdb as first argument
                        # <ligand id>-<ligand residue id> as 2nd argument and the crystal() object
                        # as 3rd argument
                        #
                        LMCSS_obj = LMCSS(LMCSS_local, ligand_info, crystal_obj)
                        #align the
                        LMCSS_obj.align_LMCSS_onto_crystal(check_point_number = 10)
                        #calculate the distance
                        LMCSS_obj.calculate_distance()

                        # The distance between the LMCSS and the crystal is set to LMCSS_distance
                        # variable and registered
                        #
                        LMCSS_distance = LMCSS_obj._min_dis
                        result_container.register(target_dir, docked_type = "LMCSS_ori", value = LMCSS_distance)
                        logger.info("\tSuccessfully calculate the distance between original LMCSS ligand vs crstal ligand. Distance is %s"%LMCSS_distance)
                    else:
                        logger.info("There are %s original LMCSS complex files, which is abnormal and need to check"%len(LMCSS_complex_only_list))
                    os.chdir(pdbid_local_path)
                except Exception as ex:
                    result_container.register(target_dir, docked_type = None, value = None)
                    logger.exception("For this folder %s, could not create the crystal object"%target_dir)
                    os.chdir(current_dir)
                    continue
                #here get the crystal obj and loop all docked structure 
                for mol_index, potential_mol in enumerate (all_mol_files):
                    # all_receptor_files contains all *docked.pdb from the target directory
                    #
                    potential_pdb = all_receptor_files[mol_index]

                    # copy both the .pdb and .mol file to the score directory
                    commands.getoutput("cp %s score"%potential_mol)                     
                    commands.getoutput("cp %s score"%potential_pdb)

                    #change path to local score folder
                    os.chdir("score")
                    local_potential_mol = os.path.basename(potential_mol)               
                    local_potential_pdb = os.path.basename(potential_pdb)

                    # sets docked_structure_type to value of PREFIX in
                    # Files with format as follows:
                    # <PREFIX>-<target id>_<candidate id>_docked.mol
                    # PREFIX should be one of the following according to these docs:
                    # LMCSS, SMCSS, hiResHolo, hiResApo
                    #
                    # See link below for more info
                    # https://github.com/drugdata/D3R/wiki/Challenge-docked-results-file-structure
                    #
                    parsed_name = re.findall('([a-zA-Z0-9]+)-([a-zA-Z0-9]+)_([a-zA-Z0-9]+)_docked.mol',
                                             local_potential_mol)
                    docked_structure_type = parsed_name[0][0]
                    try:
                        # copies local_potential_mol and local_potential_pdb to
                        # score directory and
                        # creates <PREFIX>-<target id>_<candidate id>_docked_ligand.pdb
                        # using schroding structalign
                        docked_obj = docked(local_potential_mol,
                                            local_potential_pdb,
                                            docked_structure_type,
                                            crystal_obj)

                        # creates score/<PREFIX>-<target id>_<candidate id>_docked_ligand_complex.pdb
                        # by merging <PREFIX>-<target id>_<candidate id>_docked_ligand.pdb
                        # with local_potential_pdb using merge_two_pdb() function
                        #
                        docked_obj.create_complex()

                        #
                        # TODO Continue documenting from this point
                        #
                        docked_obj.align_complex_onto_crystal(check_point_number = 10)
                        docked_obj.calculate_rmsd_and_distance()
                        (rmsd, dis) = docked_obj._min_rmsd_dis
                        docked_structure_type_dis = docked_structure_type + "_dis"
                        #rmsd = docked_obj._min_rmsd_dis[0]
                        valid_target = True
                        result_container.register(target_dir, docked_type = docked_structure_type, value = rmsd) 
                        result_container.register(target_dir, docked_type = docked_structure_type_dis, value = dis) 
                        logger.info("\tSuccessfully calculate the rmsd for this category %s, the rmsd is %s"%(docked_structure_type, rmsd))
                        #update the pickle and txt csv file if there is valid case found 
                        result_container.layout_pickle (pickle_filename = pickle_full_path)
                        result_container.layout_json (json_filename = json_full_path)
                        result_container.layout_plain (plain_filename = plain_full_path)
                        os.chdir(pdbid_local_path)
                    except Exception as ex:
                        result_container.register(target_dir, docked_type = docked_structure_type, value = None) 
                        os.chdir(pdbid_local_path)
                        logger.exception("For this type of structure: %s, the rmsd could not be calculated"%docked_structure_type)
                        continue
                logger.info("++++++++++++++++Finished for this target %s+++++++++++++++++"%target_name)
                os.chdir(current_dir)
            else:
                result_container.register(target_dir, docked_type = None, value = None)
                logger.info("For this folder %s, there is no valid crystal structure"%target_dir)
                ######need to register into the result pickle and txt, csv 
                os.chdir(current_dir)
                continue
    #after loop all possible target, layout the pickle and plain files    
    result_container.layout_pickle (pickle_filename = pickle_full_path)
    result_container.layout_json (json_filename = json_full_path)
    result_container.layout_plain (plain_filename = plain_full_path)


def main(args):
    """Main entry into script
    """
    parser = ArgumentParser()
    reqarg = parser.add_argument_group('required named arguments')

    reqarg.add_argument("-d", "--dockdir", metavar="PATH", required=True,
                        help="PATH where we could find the docking "
                             "stage output")

    reqarg.add_argument("-b", "--blastnfilterdir", metavar="PATH",
                        required=True,
                        help="PATH where we could find the blastnfilter "
                             "stage output")

    reqarg.add_argument("-c", "--challengedir", metavar="PATH", required=True,
                        help="PATH where we could find the challengedata "
                             "stage output")

    reqarg.add_argument("-o", "--outdir", metavar="PATH", required=True,
                        help="PATH where we will run the evaluate stage")

    reqarg.add_argument("-p", "--pdbdb", metavar="PATH", required=True,
                        help="PDB DATABANK which we will "
                             "get the crystal structure")

    opt = parser.parse_args(args[1:])

    logging.basicConfig(format='%(asctime)s: %(message)s',
                        datefmt='%m/%d/%y %I:%M:%S',
                        filename=os.path.join(opt.outdir, FINAL_LOG),
                        filemode='w', level=logging.DEBUG)

    logger.debug('Calling main_score function')
    main_score(opt.dockdir, opt.pdbdb, opt.outdir,
               opt.blastnfilterdir, opt.challengedir)
    return 0


if __name__ == '__main__':  # pragma: no cover
    sys.exit(main(sys.argv))




__author__ = 'churas'

import unittest
import tempfile
import os.path

from d3r.filter.filter import QueryFilter
"""
test_in_put
--------------------------------

Tests for `in_put` module.
"""

import shutil
from d3r.utilities.analysis import OutputAnalysis


class TestAnalysis(unittest.TestCase):
    def setUp(self):
        pass

    def test_print_filter_criteria(self):
        temp_dir = tempfile.mkdtemp()
        try:
            oa = OutputAnalysis()
            oa.print_filter_criteria(temp_dir)
            f = open(os.path.join(temp_dir, 'summary.txt'), 'r')
            lines = f.readlines()
            f.close()
            self.assertTrue('FILTERING CRITERIA' in lines[0])
            self.assertTrue('No. of query sequences           <=    ' +
                            str(QueryFilter.sequence_threshold) in lines[1])
            self.assertTrue('No. of dockable ligands           =    ' +
                            str(QueryFilter.dockable_ligand_threshold) in
                            lines[2])
            self.assertTrue('Ligand self symmetries' in lines[7])
            self.assertTrue('<=    100' in lines[7])
        finally:
            shutil.rmtree(temp_dir)

    def tearDown(self):
        pass


if __name__ == '__main__':
    unittest.main()
